@@ -1,10 +1,11 @@
 /*
  * tools/e2fsadm.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Germany
  *
  * June,September 1998
- * February 1999
+ * February,July,October 1999
+ * January,February 2000
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,10 +30,23 @@
  *    04/09/1998 - corrected some messages
  *    06/02/1999 - fixed size bug
  *               - fixed lvm_check_number() usage
+ *    23/07/1999 - fixed modulo size bug
+ *    25/07/1999 - supported ext2fs with blocksize != 1k
+ *    21/09/1999 - added environment variable support for resizer command
+ *                 and options (idea by Torsten Neumann
+ *                 <londo!torsten@odb.rhein-main.de>)
+ *    06/10/1999 - implemented support for long options
+ *    01/01/2000 - take care of rounding if striped resize
+ *                 (idea by Pete Allen <petea3@attglobal.net>)
+ *    15/02/2000 - use lvm_error()
+ *    17/07/2000 - Add support for ext2online <adilger@turbolinux.com>
+ *    18/07/2000 - use getmntent on /etc/mtab, then /proc/mounts for mounted LV
+ *               - also check mountpoint in addition to device name.
  *
  */
 
 #include <lvm_user.h>
+#include <mntent.h>
 
 char *cmd = NULL;
 
@@ -43,12 +57,15 @@ int opt_d = 0;
 typedef struct {
    uint32_t	dummy1;
    uint32_t	blocks_count;
-   uint32_t	dummy2[12];
+   uint32_t	dummy2[4];
+   uint32_t	block_size;
+   uint32_t	dummy3[7];
    uint16_t	s_magic;
-   uint32_t	dummy3[454];
+   uint32_t	dummy4[454];
 } ext2_super_t;
 
-static char *bin_dirs[] = { "/sbin", "/usr/sbin", "/bin", "/usr/bin", NULL};
+static char *bin_dirs[] = { "/sbin", "/usr/sbin", "/usr/local/sbin", "/bin", "/usr/bin", "/usr/local/bin", NULL};
+static char *mnt_files[] = { MOUNTED, "/proc/mounts", MNTTAB , NULL};
 
 /* internal function prototypes */
 char *lvm_find_command ( char *);
@@ -60,7 +77,6 @@ int main ( int argc, char **argv) {
    int c = 0;
    int grow = UNDEF;
    int l = 0;
-   int len = 0;
    int opt_l = 0;
    int opt_L = 0;
    int opt_n = 0;
@@ -68,29 +84,47 @@ int main ( int argc, char **argv) {
    int opt_v = 0;
    int ret = 0;
    int sign = 0;
+   int size_rest = 0;
    int size_equal = FALSE;
    int size_invalid = FALSE;
    long size = 0;
    char *cmd_fsck = "e2fsck";
-   char *cmd_resize = "resize2fs";
+   char *cmd_resize = NULL;
+   char *cmd_resize_opts = NULL;
+   char *cmd_resize_default = "resize2fs";
+   char *cmd_resize_opts_default = "-p";
+   char *cmd_resize_default2 = "ext2resize";
+   char *cmd_resize_opts_default2 = "-v";
+   char *cmd_online = NULL;
+   char *cmd_online_default = "ext2online";
+   char *cmd_online_opts = NULL;
+   char *cmd_online_opts_default = "-v";
    char *cmd_extend = "lvextend";
    char *cmd_ptr = NULL;
    char *cmd_reduce = "lvreduce";
    char *dummy = NULL;
    char *lv_name = NULL;
-   char *mount = NULL;
    char *vg_name = NULL;
-   char *proc_mounts = "/proc/mounts";
-   char *ptr = NULL;
+   char **mntfile = NULL;
    char *size_ptr = NULL;
-   char buffer[2*NAME_LEN] = { 0, };
-   char command[2*NAME_LEN] = { 0, };
+   char command[2*NAME_LEN];
 #ifdef DEBUG
-   char *options = "dh?L:l:ntv";
+   char *options = "dh?l:L:ntv";
 #else
-   char *options = "h?L:l:ntv";
+   char *options = "h?l:L:ntv";
 #endif
-   FILE *proc = NULL;
+   struct option long_options[] = {
+#ifdef DEBUG
+      { "debug",   no_argument,       NULL, 'd'},
+#endif
+      { "help",    no_argument,       NULL, 'h'},
+      { "extents", required_argument, NULL, 'l'},
+      { "size",    required_argument, NULL, 'L'},
+      { "nofsck",  no_argument,       NULL, 'n'},
+      { "test",    no_argument,       NULL, 't'},
+      { "verbose", no_argument,       NULL, 'v'},
+      { NULL,      0,                 NULL, 0}
+   };
    lv_t *lv = NULL;
    vg_t *vg = NULL;
    ext2_super_t *super = NULL;
@@ -101,7 +135,8 @@ int main ( int argc, char **argv) {
    SUSER_CHECK;
    LVMTAB_CHECK;
 
-   while ( ( c = getopt ( argc, argv, options)) != EOF) {
+   while ( ( c = getopt_long ( argc, argv, options,
+                               long_options, NULL)) != EOF) {
       switch ( c) {
 #ifdef DEBUG
          case 'd':
@@ -115,21 +150,21 @@ int main ( int argc, char **argv) {
 
          case 'h':
          case '?':
-            printf ( "\n%s\n\n%s -- ext2 Filesystem and LVM Administration\n\n"
+            printf ( "\n%s (IOP %d)\n\n%s -- ext2 Filesystem and LVM Administration\n\n"
                      "Synopsis:\n"
                      "---------\n\n"
                      "%s\n"
 #ifdef DEBUG
-                     "\t[-d]\n"
+                     "\t[-d/--debug]\n"
 #endif
-                     "\t[-h/?]\n"
-                     "\t{ -l [+/-]LogicalExtentsNumber |\n"
-                     "\t  -L [+/-]LogicalVolumeSize[kKmMgGtT]}\n"
-                     "\t[-n]\n"
-                     "\t[-t]\n"
-                     "\t[-v]\n"
+                     "\t[-h/-?/--debug]\n"
+                     "\t{ -l/--extents [+/-]LogicalExtentsNumber |\n"
+                     "\t  -L/--size [+/-]LogicalVolumeSize[kKmMgGtT]}\n"
+                     "\t[-n/--nofsck]\n"
+                     "\t[-t/--test]\n"
+                     "\t[-v/--verbose]\n"
                      "\tLogicalVolumePath\n\n",
-                     lvm_version, cmd, cmd);
+                     lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd);
             return 0;
             break;
 
@@ -186,11 +221,28 @@ int main ( int argc, char **argv) {
 
    CMD_MINUS_CHK;
 
-   if ( lvm_find_command ( cmd_resize) == NULL) {
-      lvm_path_error ( cmd_resize, "resize");
-      return LVM_EE2FSADM_RESIZE_PATH;
+   if ( ( cmd_resize = getenv ( "E2FSADM_RESIZE_CMD" )) == NULL) {
+      if ( lvm_find_command ( cmd_resize_default)) {
+         cmd_resize = cmd_resize_default;
+         cmd_resize_opts = cmd_resize_opts_default;
+      } else if ( lvm_find_command ( cmd_resize_default2)) {
+         cmd_resize = cmd_resize_default2;
+         cmd_resize_opts = cmd_resize_opts_default2;
+      }
    }
 
+   if ( ( dummy = getenv ( "E2FSADM_RESIZE_OPTS" )))
+      cmd_online_opts = dummy;
+
+   if ( ( cmd_online = getenv ( "E2FSADM_ONLINE_CMD" )) == NULL) {
+      cmd_online = cmd_online_default;
+      cmd_online_opts = cmd_online_opts_default;
+   }
+
+   if ( ( dummy = getenv ( "E2FSADM_ONLINE_OPTS" )))
+      cmd_online_opts = dummy;
+
+
    if ( opt_l == 0 && opt_L == 0) {
       fprintf ( stderr, "%s -- please give either an l or an L option\n\n",
                         cmd);
@@ -203,11 +255,48 @@ int main ( int argc, char **argv) {
       return LVM_EE2FSADM_LV_MISSING;
    }
 
-   lv_name = argv[optind];
-
    if ( opt_t > 0) printf ( "%s -- this is a test run and no real resize\n",
                             cmd);
 
+   lv_name = argv[optind];
+
+   /* check to see if the LV or mountpoint appears in one of the mtab files */
+   for (mntfile = mnt_files; *mntfile != NULL; mntfile++) {
+      struct mntent *mntent;
+      FILE *mounts;
+
+      if ( ( mounts = setmntent ( *mntfile, "r")) == NULL)
+         continue;
+
+      while ( (mntent = getmntent ( mounts)) != NULL) {
+         /* Also check the mount point, and update lv_name if it matches */
+         if (strcmp (lv_name, mntent->mnt_dir) == 0)
+            lv_name = mntent->mnt_fsname;
+         if ( strcmp ( lv_name, mntent->mnt_fsname) == 0) {
+            /* If we aren't looking at /etc/fstab, the filesystem is mounted */
+            if (strcmp(*mntfile, MNTTAB)) {
+               if ( lvm_find_command ( cmd_online) == NULL) {
+                  lvm_path_error ( cmd_online, "online resize");
+                  fprintf ( stderr, "%s -- ERROR: '%s' is mounted on '%s'.\n"
+                                    "%s -- please umount it to resize\n\n",
+                                    cmd, lv_name, mntent->mnt_dir, cmd);
+                  endmntent(mounts);
+                  return LVM_EE2FSADM_RESIZE_PATH;
+               }
+               /* FIXME we could check the mnt_type for reiserfs here and
+                *       set cmd_resize to resize_reiserfs and its options.
+                */
+               cmd_resize = cmd_online;
+               cmd_resize_opts = cmd_online_opts;
+            }
+            /* don't close mounts, in case we point to mnt_fsname */
+            goto have_mntent;
+         }
+      }
+      endmntent(mounts);
+   }
+have_mntent:
+
    if ( opt_v > 0) printf ( "%s -- checking logical volume name\n", cmd);
    if ( lv_check_name ( lv_name) < 0) {
       fprintf ( stderr, "%s -- invalid logical volume name \"%s\"\n\n",
@@ -295,45 +384,30 @@ int main ( int argc, char **argv) {
       printf ( "%s -- correcting size %s to physical extent boundary ",
                cmd, ( dummy = lvm_show_size ( size / 2, SHORT)));
       free ( dummy);
-      size += vg->pe_size + ( size % vg->pe_size);
+      size += vg->pe_size - ( size % vg->pe_size);
+      if ( size > 0 && sign == -1) size -= vg->pe_size;
       printf ( "%s\n", ( dummy = lvm_show_size ( size / 2, SHORT)));
       free ( dummy); dummy = NULL;
    }
 
-   if ( ( proc = fopen ( proc_mounts, "r")) == NULL) {
-      fprintf ( stderr, "%s -- ERROR: can't open \"%s\"\n\n",
-                        cmd, proc_mounts);
-      return LVM_EE2FSADM_PROC_MOUNTS;
+   size_rest = size % ( vg->lv[l]->lv_stripes * vg->pe_size);
+   if ( size_rest != 0) {
+      printf ( "%s -- rounding size %ld KB to stripe boundary size ",
+               cmd, size / 2);
+      size = size - size_rest + ( vg->lv[l]->lv_stripes * vg->pe_size);
+      printf ( "%ld KB\n", size / 2);
    }
 
-   len = strlen ( lv_name);
-   while ( fgets ( buffer, sizeof ( buffer), proc) != NULL) {
-      if ( strncmp ( lv_name, buffer, len) == 0) {
-         mount = buffer;
-         while ( *mount != ' ') mount++;
-         mount++;
-         ptr = mount;
-         while ( *ptr != ' ') ptr++;
-         *ptr = 0;
-         fprintf ( stderr, "%s -- ERROR: \"%s\" is mounted on \"%s\"\n"
-                           "%s -- please umount it to resize\n\n",
-                           cmd, lv_name, mount, cmd);
-         fclose ( proc);
-         return LVM_EE2FSADM_MOUNTED;
-      }
-   }
-   fclose ( proc);
-
    if ( ( super = lvm_is_ext2_fs ( lv_name)) == NULL)
       return LVM_EE2FSADM_NO_EXT2;
 
-   if ( super->blocks_count + sign * size / 2 < 0) {
+   if ( ( super->blocks_count<<super->block_size) + sign * size / 2 < 0) {
       fprintf ( stderr, "%s -- logical volume size for \"%s\" invalid\n\n",
                         cmd, lv_name);
       return LVM_EE2FSADM_LV_SIZE;
    }
 
-   if ( opt_n == 0) {
+   if ( opt_n == 0 && cmd_resize != cmd_online) {
       if ( opt_v > 0) printf ( "%s -- checking filesystem (this can "
                                "take a while)\n", cmd);
       if ( ( cmd_ptr = lvm_find_command ( cmd_fsck)) == NULL) {
@@ -341,7 +415,7 @@ int main ( int argc, char **argv) {
          return LVM_EE2FSADM_FSCK_PATH;
       }
       if ( opt_t == 0) {
-         sprintf ( command, "%s -f %s%c", cmd_ptr, lv_name, 0);
+         sprintf ( command, "%s -f %s", cmd_ptr, lv_name);
          if ( system ( command) != 0) return  LVM_EE2FSADM_FSCK_RUN;
       }
    }
@@ -353,8 +427,8 @@ int main ( int argc, char **argv) {
          lvm_path_error ( cmd_extend, "extend logical volume");
          return LVM_EE2FSADM_LV_EXTEND_PATH;
       }
-      sprintf ( command, "%s -l %lu %s%c",
-                         cmd_ptr, size / vg->pe_size, lv_name, 0);
+      sprintf ( command, "%s -l %lu %s",
+                         cmd_ptr, size / vg->pe_size, lv_name);
       if ( opt_t == 0) {
          if ( system ( command) != 0) return LVM_EE2FSADM_LV_EXTEND_RUN;
          if ( ( ret = lvm_tab_lv_read_by_name ( vg_name, lv_name, &lv)) < 0) {
@@ -372,17 +446,18 @@ int main ( int argc, char **argv) {
       }
       if ( opt_v > 0) printf ( "%s -- resizing filesystem (this can "
                                "take a while)\n", cmd);
-      if ( ( cmd_ptr = lvm_find_command ( cmd_resize)) == NULL) {
-         lvm_path_error ( cmd_resize, "resize");
+      if ( !cmd_resize || ( cmd_ptr = lvm_find_command ( cmd_resize)) == NULL) {
+         lvm_path_error(cmd_resize ? cmd_resize : cmd_resize_default, "resize");
          return LVM_EE2FSADM_RESIZE_PATH;
       }
-      sprintf ( command, "%s -p %s %lu%c",
-                         cmd_ptr, lv_name, size / 2, 0);
+      sprintf ( command, "%s %s %s %lu",
+                         cmd_ptr, cmd_resize_opts,
+                         lv_name, ( size / 2)>>super->block_size);
       if ( opt_t == 0) {
          if ( system ( command) != 0) return LVM_EE2FSADM_RESIZE_RUN;
          if ( ( super = lvm_is_ext2_fs ( lv_name)) == NULL)
             return LVM_EE2FSADM_NO_EXT2;
-         if ( size / 2 != super->blocks_count) {
+         if ( ( size / 2)>>super->block_size != super->blocks_count) {
             fprintf ( stderr, "%s -- failure of of \"%s\": \"%s\" "
                               "was not successfully extended\n",
                               cmd, cmd_ptr, lv_name);
@@ -393,17 +468,18 @@ int main ( int argc, char **argv) {
    } else { /* grow == FALSE */
       if ( opt_v > 0) printf ( "%s -- resizing filesystem (this can "
                                "take a while)\n", cmd);
-      if ( ( cmd_ptr = lvm_find_command ( cmd_resize)) == NULL) {
-         lvm_path_error ( cmd_resize, "resize");
+      if ( !cmd_resize || ( cmd_ptr = lvm_find_command ( cmd_resize)) == NULL) {
+         lvm_path_error(cmd_resize ? cmd_resize : cmd_resize_default, "resize");
          return LVM_EE2FSADM_RESIZE_PATH;
       }
-      sprintf ( command, "%s -p %s %lu%c",
-                         cmd_ptr, lv_name, size / 2, 0);
+      sprintf ( command, "%s %s %s %lu",
+                         cmd_ptr, cmd_resize_opts,
+                         lv_name, ( size / 2)>>super->block_size);
       if ( opt_t == 0) {
          if ( system ( command) != 0) return LVM_EE2FSADM_RESIZE_RUN;
          if ( ( super = lvm_is_ext2_fs ( lv_name)) == NULL)
             return LVM_EE2FSADM_NO_EXT2;
-         if ( size / 2 != super->blocks_count) {
+         if ( ( size / 2)>>super->block_size != super->blocks_count) {
             fprintf ( stderr, "%s -- failure of of \"%s\": \"%s\" "
                               "was not successfully reduced\n",
                               cmd, cmd_ptr, lv_name);
@@ -415,8 +491,8 @@ int main ( int argc, char **argv) {
          lvm_path_error ( cmd_reduce, "reduce logical volume");
          return LVM_EE2FSADM_LV_REDUCE_PATH;
       }
-      sprintf ( command, "%s -f -l %lu %s%c",
-                         cmd_ptr, size / vg->pe_size, lv_name, 0);
+      sprintf ( command, "%s -f -l %lu %s",
+                         cmd_ptr, size / vg->pe_size, lv_name);
       if ( opt_t == 0) {
          if ( system ( command) != 0) return LVM_EE2FSADM_LV_REDUCE_RUN;
       }
@@ -439,7 +515,7 @@ char *lvm_find_command ( char *cmd_to_find) {
    if ( cmd_to_find == NULL || strlen ( cmd_to_find) == 0) return NULL;
 
    while ( bin_dirs[i] != NULL) {
-      sprintf ( command, "%s/%s%c", bin_dirs[i], cmd_to_find, 0);
+      sprintf ( command, "%s/%s", bin_dirs[i], cmd_to_find);
       if ( stat ( command, &stat_b) == 0) return command;
       i++;
    }
@@ -469,22 +545,23 @@ ext2_super_t *lvm_is_ext2_fs ( char *device) {
    ext2_super_t *ret = &super;
 
    if ( ( fs = open ( device, O_RDONLY)) == -1) {
-      fprintf ( stderr, "%s -- ERROR %d opening logical volume \"%s\"\n\n",
-                        cmd, errno, device);
+      fprintf ( stderr, "%s -- ERROR \"%s\" opening logical volume \"%s\"\n\n",
+                        cmd, lvm_error ( errno), device);
       ret = NULL;
       goto lvm_is_ext2_fs_end;
    }
 
    if ( lseek ( fs, 1024, SEEK_SET) == -1) {
-      fprintf ( stderr, "%s -- ERROR %d seeking for superblock on \"%s\"\n\n",
-                        cmd, errno, device);
+      fprintf ( stderr, "%s -- ERROR \"%s\" seeking for superblock "
+                        "on \"%s\"\n\n",
+                        cmd, lvm_error ( errno), device);
       ret = NULL;
       goto lvm_is_ext2_fs_end;
    }
 
    if ( read ( fs, &super, sizeof ( super)) != sizeof ( super)) {
-      fprintf ( stderr, "%s -- ERROR %d reading superblock from \"%s\"\n\n",
-                        cmd, errno, device);
+      fprintf ( stderr, "%s -- ERROR \"%s\" reading superblock from \"%s\"\n\n",
+                        cmd, lvm_error ( errno), device);
       ret = NULL;
       goto lvm_is_ext2_fs_end;
    }
