@@ -5,6 +5,7 @@
  *
  * May,August,November 1998
  * February,April 1999
+ * January 2000
  *
  *
  * This LVM library is free software; you can redistribute it and/or
@@ -27,12 +28,13 @@
 /*
  * Changelog
  *
- *   30/08/1998 - checked for direct MAJOR due to seperation of disk
- *                and core structures
- *   12/11/1998 - added new SCSI majors
- *   21/02/1999 - changed to check in loop
- *   17/03/1999 - added loop device support
- *   22/04/1999 - DAC960 support by Matthew Taylor <M.Taylor@rbgkew.org.uk>
+ *    30/08/1998 - checked for direct MAJOR due to seperation of disk
+ *                 and core structures
+ *    12/11/1998 - added new SCSI majors
+ *    21/02/1999 - changed to check in loop
+ *    17/03/1999 - added loop device support
+ *    22/04/1999 - DAC960 support by Matthew Taylor <M.Taylor@rbgkew.org.uk>
+ *    31/01/2000 - use debug_enter()/debug_leave()
  *
  */
 
@@ -66,6 +68,13 @@ int lvm_check_dev ( struct stat *stat_b, int check_mode) {
 #endif
 #ifdef	DAC960_MAJOR
       DAC960_MAJOR,
+      DAC960_MAJOR+1,
+      DAC960_MAJOR+2,
+      DAC960_MAJOR+3,
+      DAC960_MAJOR+4,
+      DAC960_MAJOR+5,
+      DAC960_MAJOR+6,
+      DAC960_MAJOR+7,
 #endif
 #ifdef LOOP_MAJOR
       LOOP_MAJOR,
@@ -75,27 +84,58 @@ int lvm_check_dev ( struct stat *stat_b, int check_mode) {
 #endif
 #ifdef NBD_MAJOR
       NBD_MAJOR,
+#endif
+#ifdef	COMPAQ_SMART2_MAJOR
+      COMPAQ_SMART2_MAJOR,
+#endif
+#ifdef	COMPAQ_SMART2_MAJOR1
+      COMPAQ_SMART2_MAJOR1,
+#endif
+#ifdef	COMPAQ_SMART2_MAJOR2
+      COMPAQ_SMART2_MAJOR2,
+#endif
+#ifdef	COMPAQ_SMART2_MAJOR3
+      COMPAQ_SMART2_MAJOR3,
+#endif
+#ifdef	COMPAQ_SMART2_MAJOR4
+      COMPAQ_SMART2_MAJOR4,
+#endif
+#ifdef	COMPAQ_SMART2_MAJOR5
+      COMPAQ_SMART2_MAJOR5,
+#endif
+#ifdef	COMPAQ_SMART2_MAJOR6
+      COMPAQ_SMART2_MAJOR6,
+#endif
+#ifdef	COMPAQ_SMART2_MAJOR7
+      COMPAQ_SMART2_MAJOR7,
 #endif
       -1
    };
 
 
 
+#ifdef DEBUG
+   debug_enter ( "lvm_check_dev -- CALLED\n");
+#endif
+
    if ( stat_b == NULL ||
-        ( check_mode != TRUE && check_mode != FALSE) ) return -LVM_EPARAM;
-
-   major = MAJOR ( stat_b->st_rdev);
-
-   for ( i = 0; valid_majors[i] != -1; i++) {
-      if (  stat_b->st_rdev == valid_majors[i] ||
-            major           == valid_majors[i]) {
-         ret = TRUE;
-         break;
+        ( check_mode != TRUE && check_mode != FALSE) ) ret = -LVM_EPARAM;
+   else {
+      major = MAJOR ( stat_b->st_rdev);
+      for ( i = 0; valid_majors[i] != -1; i++) {
+         if (  stat_b->st_rdev == valid_majors[i] ||
+               major           == valid_majors[i]) {
+            ret = TRUE;
+            break;
+         }
       }
    }
 
    if ( ret == TRUE && check_mode == TRUE && ! S_ISBLK ( stat_b->st_mode))
       ret = FALSE;
 
+#ifdef DEBUG
+   debug_leave ( "lvm_check_dev -- LEAVING with ret: %d\n", ret);
+#endif
    return ret;
 }
