@@ -1,9 +1,10 @@
 /*
  * tools/lib/lvm_show_filetype.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Germany
  *
  * May 1998
+ * January 2000
  *
  *
  * This LVM library is free software; you can redistribute it and/or
@@ -23,20 +24,37 @@
  *
  */
 
+/*
+ * Changelog
+ *
+ *    31/01/2000 - use debug_enter()/debug_leave()
+ *
+ */
+
 #include <liblvm.h>
 
 int lvm_show_filetype ( ushort mode, char *name) {
+   int ret = 0;
    char *what_file = "file";
 
-   if ( name == NULL) return -LVM_EPARAM;
+#ifdef DEBUG
+   debug_enter ( "lvm_show_filetype -- CALLED\n");
+#endif
 
-   if ( S_ISLNK  ( mode)) what_file = "symlink";
-   if ( S_ISDIR  ( mode)) what_file = "directory";
-   if ( S_ISCHR  ( mode)) what_file = "character special";
-   if ( S_ISBLK  ( mode)) what_file = "block special";
-   if ( S_ISFIFO ( mode)) what_file = "fifo";
-   if ( S_ISSOCK ( mode)) what_file = "socket";
-   fprintf ( stderr, "%s -- %s %s allready exists\n\n",
-                     cmd, what_file, name);
-   return 0;
+   if ( name == NULL) ret = -LVM_EPARAM;
+   else {
+      if ( S_ISLNK  ( mode)) what_file = "symlink";
+      if ( S_ISDIR  ( mode)) what_file = "directory";
+      if ( S_ISCHR  ( mode)) what_file = "character special";
+      if ( S_ISBLK  ( mode)) what_file = "block special";
+      if ( S_ISFIFO ( mode)) what_file = "fifo";
+      if ( S_ISSOCK ( mode)) what_file = "socket";
+      fprintf ( stderr, "%s -- %s %s allready exists\n\n",
+                        cmd, what_file, name);
+   }
+
+#ifdef DEBUG
+   debug_leave ( "lvm_show_filetype -- LEAVING\n");
+#endif
+   return ret;
 }
