@@ -1,9 +1,10 @@
 /*
  * tools/lib/pv_check_volume.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Germany
  *
  * March 1997
+ * February 2000
  *
  *
  * This LVM library is free software; you can redistribute it and/or
@@ -23,26 +24,31 @@
  *
  */
 
+/*
+ * Changelog
+ *
+ *    31/01/2000 - use debug_enter()/debug_leave()
+ *
+ */
+
 #include <liblvm.h>
 
 int pv_check_volume ( char *dev_name, pv_t *pv) {
    int ret = FALSE;
 
 #ifdef DEBUG
-   debug ( "pv_check_volume -- CALLED  dev_name: \"%s\"  pv: %0X\n",
-            dev_name, ( uint) pv);
+   debug_enter ( "pv_check_volume -- CALLED  dev_name: \"%s\"  pv: %0X\n",
+                 dev_name, ( uint) pv);
 #endif
 
    if ( dev_name == NULL ||
         pv_check_name ( dev_name) < 0 ||
-        pv == NULL) return -LVM_EPARAM;
-
-   if ( /* strcmp  ( dev_name, pv->pv_name) == 0 && */
-        ( pv_check_new ( pv) == TRUE ||
-          vg_check_name ( pv->vg_name) == 0)) ret = TRUE;
+        pv == NULL) ret = -LVM_EPARAM;
+   else if ( pv_check_new ( pv) == TRUE ||
+             vg_check_name ( pv->vg_name) == 0) ret = TRUE;
 
 #ifdef DEBUG
-   debug ( "pv_check_volume -- LEAVING with ret: %d\n", ret);
+   debug_leave ( "pv_check_volume -- LEAVING with ret: %d\n", ret);
 #endif
    return ret;
 }
