@@ -5,7 +5,8 @@
  *
  * May  1998
  * July 1998
- * January,February 1999
+ * January,February,September,October 1999
+ * January,February 2000
  *
  *
  * This LVM library is free software; you can redistribute it and/or
@@ -39,6 +40,14 @@
  *    20/01/1999 - used LVM_PE_DISK_OFFSET macro in pv_move_pe()
  *    06/02/1999 - fixed lv_current_le corection bug
  *    16/02/1999 - changed to new pe_lock()
+ *    26/08/1999 - support for loop devices (Johannes Deisenhofer
+ *                                           jo@hexmac.com)
+ *    29/10/1999 - fixed possible free() bug
+ *    01/10/1999 - fixed offset by one mistake in case of striped logical
+ *                 volumes while checking if logical volume already has extents
+ *                 on a destination physical volume
+ *    03/02/2000 - use debug_enter()/debug_leave()
+ *    15/02/2000 - use lvm_error()
  *
  */
 
@@ -60,7 +69,7 @@ static lv_t *lv_this = NULL;
 /* perform the move of all physical extents / those of a logical volume */
 int pv_move_pes ( vg_t *vg, char *buffer, char **pv_allowed,
                   int src_pv_index, int lv_index,
-                  int *les, int *source_pes, int *dest_pes,
+                  long *les, long *source_pes, long *dest_pes,
                   int opt_v, int opt_t) {
    int add_it = FALSE;
    int dest_pe_count = 0;
@@ -86,7 +95,7 @@ int pv_move_pes ( vg_t *vg, char *buffer, char **pv_allowed,
 
 
 #ifdef DEBUG
-   debug ( "pv_move_pes -- CALLED\n");
+   debug_enter ( "pv_move_pes -- CALLED\n");
 #endif
 
    if ( vg == NULL || vg_check_consistency ( vg) < 0 ||
@@ -113,9 +122,10 @@ int pv_move_pes ( vg_t *vg, char *buffer, char **pv_allowed,
       source_pe_count = 0;
       for ( ext = 0; source_pes[ext] != -1; ext++) {
          if ( source_pes[ext] < 0 ||
-              source_pes[ext] >= vg->pv[src_pv_index]->pe_total)
-            return -LVM_EPARAM;
-         else source_pe_count++;
+              source_pes[ext] >= vg->pv[src_pv_index]->pe_total) {
+            ret = -LVM_EPARAM;
+            goto pv_move_pes_end;
+         } else source_pe_count++;
       }
    }
 
@@ -125,13 +135,17 @@ int pv_move_pes ( vg_t *vg, char *buffer, char **pv_allowed,
       for ( ext = 0; dest_pes[ext] != -1; ext++) {
          if ( dest_pes[ext] < 0 ||
               dest_pes[ext] >= vg->pv[dst_pv_index]->pe_total) {
-            return -LVM_EPARAM;
+            ret = -LVM_EPARAM;
+            goto pv_move_pes_end;
          } else dest_pe_count++;
       }
    }
 
    if ( source_pes != NULL && dest_pes != NULL &&
-        source_pe_count != dest_pe_count) return -LVM_EPARAM;
+        source_pe_count != dest_pe_count) {
+      ret = -LVM_EPARAM;
+      goto pv_move_pes_end;
+   }
    /* end of parameter check */
 
    pv_move_int = FALSE;
@@ -183,7 +197,7 @@ int pv_move_pes ( vg_t *vg, char *buffer, char **pv_allowed,
             if ( pe_src_sav != NULL) free ( pe_src_sav);
             ret = -LVM_EPV_MOVE_PES_REALLOC;
             goto pv_move_pes_end;
-         }
+         } else pe_src_sav = NULL;
 
          pe_src[pe_to_move].p  = src_pv_index;
          pe_src[pe_to_move].pe = pe;
@@ -254,7 +268,7 @@ int pv_move_pes ( vg_t *vg, char *buffer, char **pv_allowed,
 
       for ( pe = 0; pe < pe_to_move; pe++) {
          if ( lv_check_on_pv ( vg->pv[dst_pv_index],
-                               pe_src[pe].lv_num - 1) == TRUE &&
+                               pe_src[pe].lv_num) == TRUE &&
               vg->lv[pe_src[pe].lv_num-1]->lv_stripes > 1) break;
          if ( ( vg->lv[pe_src[pe].lv_num-1]->lv_allocation & LV_CONTIGUOUS) &&
               pv_check_free_contiguous ( vg->pv[dst_pv_index],
@@ -281,7 +295,7 @@ int pv_move_pes ( vg_t *vg, char *buffer, char **pv_allowed,
             if ( pe_dest_sav != NULL) free ( pe_dest_sav);
             ret = -LVM_EPV_MOVE_PES_REALLOC;
             goto pv_move_pes_end;
-         }
+         } else pe_dest_sav = NULL;
          pe_dest[pe_for_dest].p = dst_pv_index;
          pe_dest[pe_for_dest].pe = pe;
          pe_for_dest++;
@@ -313,7 +327,8 @@ int pv_move_pes ( vg_t *vg, char *buffer, char **pv_allowed,
                                 pe_src[pe].p,  pe_dest[pe].p,
                                 pe_src[pe].pe, pe_dest[pe].pe,
                                 opt_v, opt_t, pe + 1, pe_to_move)) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d pv_move_pe\n\n", cmd, ret);
+         fprintf ( stderr, "%s -- ERROR \"%s\" pv_move_pe\n\n",
+                           cmd, lvm_error ( ret));
          goto pv_move_pes_end;
       }
    }
@@ -325,11 +340,11 @@ pv_move_pes_end:
    if ( ret >= 0) ret = pe;
 
 #ifdef DEBUG
-   debug ( "pv_move_pes -- LEAVING\n");
+   debug_leave ( "pv_move_pes -- LEAVING with ret: %d\n", ret);
 #endif
 
    return ret;
-}
+} /* pv_move_pes() */
 
 
 /* perform the move of a physical extent */
@@ -349,17 +364,24 @@ int pv_move_pe ( vg_t *vg, char *buffer,
    size_t size = 0;
    le_remap_req_t le_remap_req;
 
+#ifdef DEBUG
+   debug_enter ( "pv_move_pe -- CALLED\n");
+#endif
+
    if ( src_pv_index < 0 || src_pv_index >= vg->pv_cur ||
         dst_pv_index < 0 || dst_pv_index >= vg->pv_cur ||
         pe_source < 0 || pe_source >= vg->pv[src_pv_index]->pe_total ||
         pe_dest   < 0 || pe_dest   >= vg->pv[dst_pv_index]->pe_total ||
         opt_v < 0 || opt_t < 0 || act_pe < 0 || off_pe < 0 ||
-        vg->pv[dst_pv_index]->pe[pe_dest].lv_num != 0) return -LVM_EPARAM;
+        vg->pv[dst_pv_index]->pe[pe_dest].lv_num != 0) {
+      ret = -LVM_EPARAM;
+      goto pv_move_pe_end;
+   }
 
    if ( ( in = open ( vg->pv[src_pv_index]->pv_name, O_RDONLY)) == -1) {
       fprintf ( stderr, "%s -- couldn't open input physical volume %s\n\n",
                 cmd, vg->pv[src_pv_index]->pv_name);
-      ret = -LVM_EPE_MOVE_PE_OPEN_IN;
+      ret = -LVM_EPV_MOVE_PE_OPEN_IN;
       goto pv_move_pe_end;
    }
    
@@ -388,7 +410,7 @@ int pv_move_pe ( vg_t *vg, char *buffer,
 
    if ( ( lv_name_this = lv_get_name ( vg, vg->pv[dst_pv_index]->\
                                            pe[pe_dest].lv_num-1)) == NULL) {
-      ret = -LVM_EPE_MOVE_PE_LV_GET_NAME;
+      ret = -LVM_EPV_MOVE_PE_LV_GET_NAME;
       goto pv_move_pe_end;
    }
 
@@ -415,7 +437,7 @@ pv_move_pe_continue:
 
    if ( opt_v > 1) {
       printf ( "lv: %s[%d]  old_dev: %02d:%02d  new_dev: %02d:%02d  "
-               "old_pe_sector: %lu  new_pe_sector: %lu\n",
+               "old_pe_sector: %u  new_pe_sector: %u\n",
                le_remap_req.lv_name,
                vg->pv[dst_pv_index]->pe[pe_dest].lv_num,
                MAJOR ( le_remap_req.old_dev),
@@ -433,7 +455,7 @@ pv_move_pe_continue:
       fprintf ( stderr, "%s -- couldn't open output "
                         "physical volume \"%s\"\n\n",
                 cmd, vg->pv[dst_pv_index]->pv_name);
-      ret = -LVM_EPE_MOVE_PE_OPEN;
+      ret = -LVM_EPV_MOVE_PE_OPEN;
       goto pv_move_pe_end;
    }
 
@@ -447,7 +469,7 @@ pv_move_pe_continue:
                 ( long) ( offset >> 32),
                 ( long) ( offset & 0xFFFFFFFF),
                 vg->pv[src_pv_index]->pv_name);
-      ret = -LVM_EPE_MOVE_PE_LLSEEK_IN;
+      ret = -LVM_EPV_MOVE_PE_LLSEEK_IN;
       goto pv_move_pe_end;
    }
 
@@ -462,7 +484,7 @@ pv_move_pe_continue:
                 ( ulong) ( offset >> 32),
                 ( ulong) ( offset & 0xFFFFFFFF),
                 vg->pv[dst_pv_index]->pv_name);
-      ret = -LVM_EPE_MOVE_PE_LLSEEK_OUT;
+      ret = -LVM_EPV_MOVE_PE_LLSEEK_OUT;
       goto pv_move_pe_end;
    }
 
@@ -494,7 +516,7 @@ pv_move_pe_continue:
                              pe[pe_dest].lv_num,
                              vg->lv[vg->pv[dst_pv_index]->\
                              pe[pe_dest].lv_num-1]->lv_dev)) < 0) {
-         ret = -LVM_EPE_MOVE_PE_LOCK;
+         ret = -LVM_EPV_MOVE_PE_LOCK;
          goto pv_move_pe_end;
       }
    }
@@ -510,7 +532,7 @@ pv_move_pe_continue:
                            "physical volume \"%s\"\n\n",
                    cmd, vg->pv[src_pv_index]->pv_name);
          pe_unlock ( vg->vg_name);
-         ret = -LVM_EPE_MOVE_PE_READ_IN;
+         ret = -LVM_EPV_MOVE_PE_READ_IN;
          goto pv_move_pe_end;
       }
    }
@@ -523,15 +545,15 @@ pv_move_pe_continue:
                            "physical volume \"%s\"\n\n",
                    cmd, vg->pv[dst_pv_index]->pv_name);
          pe_unlock ( vg->vg_name);
-         ret = -LVM_EPE_MOVE_PE_WRITE_OUT;
+         ret = -LVM_EPV_MOVE_PE_WRITE_OUT;
          goto pv_move_pe_end;
       }
-      if ( fsync ( out) < 0) {
+      if ( fsync ( out) < 0  && (errno != EINVAL && errno != EROFS)) {
          fprintf ( stderr, "%s -- ERROR syncing output to "
                            "physical volume \"%s\"\n\n",
                    cmd, vg->pv[dst_pv_index]->pv_name);
          pe_unlock ( vg->vg_name);
-         ret = -LVM_EPE_MOVE_PE_WRITE_OUT;
+         ret = -LVM_EPV_MOVE_PE_WRITE_OUT;
          goto pv_move_pe_end;
       }
    }
@@ -542,9 +564,10 @@ pv_move_pe_continue:
                             "in VGDA of kernel\n", cmd);
    if ( opt_t == 0) {
       if ( ( ret = lv_le_remap ( vg, &le_remap_req)) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d remapping\n\n", cmd, ret);
+         fprintf ( stderr, "%s -- ERROR \"%s\" remapping\n\n",
+                           cmd, lvm_error ( ret));
          pe_unlock ( vg->vg_name);
-         ret = -LVM_EPE_MOVE_LV_LE_REMAP;
+         ret = -LVM_EPV_MOVE_LV_LE_REMAP;
          goto pv_move_pe_end;
       }
    }
@@ -553,11 +576,11 @@ pv_move_pe_continue:
                             cmd);
    if ( opt_t == 0) {
       if ( ( ret = pe_unlock ( vg->vg_name)) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d unlocking PE %lu on \"%s\"\n\n",
-                           cmd, ret, 
+         fprintf ( stderr, "%s -- ERROR \"%s\" unlocking PE %lu on \"%s\"\n\n",
+                           cmd, lvm_error ( ret),
                            pe_source,
                            vg->pv[src_pv_index]->pv_name);
-         ret = -LVM_EPE_MOVE_PE_UNLOCK;
+         ret = -LVM_EPV_MOVE_PE_UNLOCK;
          goto pv_move_pe_end;
       }
    }
@@ -566,10 +589,11 @@ pv_move_pe_continue:
                             "of kernel\n",
                             cmd, vg->pv[src_pv_index]->pv_name);
    if ( opt_t == 0) {
-      if ( pv_change ( vg->vg_name, vg->pv[src_pv_index]) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d changing source \"%s\"\n\n",
-                           cmd, errno,  vg->pv[src_pv_index]->pv_name);
-         ret = -LVM_EPE_MOVE_PV_CHANGE_SRC;
+      if ( ( ret = pv_change ( vg->vg_name, vg->pv[src_pv_index])) < 0) {
+         fprintf ( stderr, "%s -- ERROR \"%s\" changing source \"%s\"\n\n",
+                           cmd, lvm_error ( ret),
+                           vg->pv[src_pv_index]->pv_name);
+         ret = -LVM_EPV_MOVE_PV_CHANGE_SRC;
          goto pv_move_pe_end;
       }
    }
@@ -578,10 +602,11 @@ pv_move_pe_continue:
                             "of kernel\n",
                             cmd, vg->pv[dst_pv_index]->pv_name);
    if ( opt_t == 0) {
-      if ( pv_change ( vg->vg_name, vg->pv[dst_pv_index]) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d changing destinatiom \"%s\"\n\n",
-                           cmd, errno, vg->pv[dst_pv_index]->pv_name);
-         ret = -LVM_EPE_MOVE_PV_CHANGE_DEST;
+      if ( ( ret = pv_change ( vg->vg_name, vg->pv[dst_pv_index])) < 0) {
+         fprintf ( stderr, "%s -- ERROR \"%s\" changing destinatiom \"%s\"\n\n",
+                           cmd, lvm_error ( ret),
+                           vg->pv[dst_pv_index]->pv_name);
+         ret = -LVM_EPV_MOVE_PV_CHANGE_DEST;
          goto pv_move_pe_end;
       }
    }
@@ -592,10 +617,11 @@ pv_move_pe_continue:
    if ( opt_t == 0) {
       if ( ( ret = pv_write_with_pe ( vg->pv[src_pv_index]->pv_name,
                                       vg->pv[src_pv_index])) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d writing source "
+         fprintf ( stderr, "%s -- ERROR \"%s\" writing source "
                            "physical volume data on \"%s\"\n\n", 
-                           cmd, ret, vg->pv[src_pv_index]->pv_name);
-         ret = -LVM_EPE_MOVE_PV_PV_WRITE_WITH_PE_SRC;
+                           cmd, lvm_error ( ret),
+                           vg->pv[src_pv_index]->pv_name);
+         ret = -LVM_EPV_MOVE_PV_PV_WRITE_WITH_PE_SRC;
          goto pv_move_pe_end;
       }
    }
@@ -606,10 +632,11 @@ pv_move_pe_continue:
    if ( opt_t == 0) {
       if ( ( ret = pv_write_with_pe ( vg->pv[dst_pv_index]->pv_name,
                                       vg->pv[dst_pv_index])) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d writing destination "
+         fprintf ( stderr, "%s -- ERROR \"%s\" writing destination "
                            "physical volume data on \"%s\"\n\n", 
-                           cmd, ret, vg->pv[dst_pv_index]->pv_name);
-         ret = -LVM_EPE_MOVE_PV_PV_WRITE_WITH_PE_DEST;
+                           cmd, lvm_error ( ret),
+                           vg->pv[dst_pv_index]->pv_name);
+         ret = -LVM_EPV_MOVE_PV_PV_WRITE_WITH_PE_DEST;
          goto pv_move_pe_end;
       }
    }
@@ -617,26 +644,38 @@ pv_move_pe_continue:
 pv_move_pe_end:
    if ( in != -1) close ( in);
    if ( out != -1) close ( out);
+
+#ifdef DEBUG
+   debug_leave ( "pv_move_pe -- LEAVING with ret: %d\n", ret);
+#endif
    return ret;
-}
+} /* pv_move_pe() */
 
 
 void pv_move_interrupt ( int sig) {
    static int first = 0;
 
+#ifdef DEBUG
+   debug_enter ( "pv_move_interrupt -- CALLED\n");
+#endif
    signal ( SIGINT, pv_move_interrupt);
    if ( first != 0) {
       if ( lv_this->lv_stripes > 1) {
          printf ( "%s -- interrupt of a striped logical volume move "
                   "not possible\n",
                   cmd);
-         return;
+         goto pv_move_interrupt_end;
       } else {
          printf ( "%s -- interrupting move... Please wait.\n",
                   cmd);
          pv_move_int = TRUE;
-         return;
+         goto pv_move_interrupt_end;
       }
    } else first++;
+
+pv_move_interrupt_end:
+#ifdef DEBUG
+   debug_leave ( "pv_move_interrupt -- LEAVING\n");
+#endif
    return;
 }
