@@ -1,10 +1,11 @@
 /*
  * tools/lib/pv_read_all_pv_of_vg.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Germany
  *
  * March-May,October-November 1997
  * May,August,November 1998
+ * January 2000
  *
  *
  * This LVM library is free software; you can redistribute it and/or
@@ -24,6 +25,13 @@
  *
  */
 
+/*
+ * Changelog
+ *
+ *    03/02/2000 - use debug_enter()/debug_leave()
+ *
+ */
+
 #include <liblvm.h>
 
 
@@ -38,12 +46,16 @@ int pv_read_all_pv_of_vg ( char *vg_name, pv_t ***pv, int reread) {
    static pv_t **pv_this = NULL;
    
 #ifdef DEBUG
-   debug ( "pv_read_all_pv_of_vg -- CALLED with vg_name: \"%s\"\n", vg_name);
+   debug_enter ( "pv_read_all_pv_of_vg -- CALLED with vg_name: \"%s\"\n",
+                 vg_name);
 #endif
 
    if ( pv == NULL || vg_name == NULL ||
         ( reread != TRUE && reread != FALSE) ||
-        vg_check_name ( vg_name) < 0) return -LVM_EPARAM;
+        vg_check_name ( vg_name) < 0) {
+      ret = -LVM_EPARAM;
+      goto pv_read_all_pv_of_vg_end;
+   }
 
    *pv = NULL;
 
@@ -61,7 +73,8 @@ int pv_read_all_pv_of_vg ( char *vg_name, pv_t ***pv, int reread) {
    }
 
    if ( first == 0) {
-      if ( ( ret = pv_read_all_pv ( &pv_tmp, FALSE)) < 0) return ret;
+      if ( ( ret = pv_read_all_pv ( &pv_tmp, FALSE)) < 0)
+         goto pv_read_all_pv_of_vg_end;
    
       /* find all PVs of this VG */
    
@@ -76,13 +89,20 @@ int pv_read_all_pv_of_vg ( char *vg_name, pv_t ***pv, int reread) {
             np++;
          }
       }
-      if ( np == 0) return -LVM_EPV_READ_ALL_PV_OF_VG_NP;
-      if ( pv_number != np) return -LVM_EPV_READ_ALL_PV_OF_VG_PV_NUMBER;
+      if ( np == 0) {
+         ret = -LVM_EPV_READ_ALL_PV_OF_VG_NP;
+         goto pv_read_all_pv_of_vg_end;
+      }
+      if ( pv_number != np) {
+         ret = -LVM_EPV_READ_ALL_PV_OF_VG_PV_NUMBER;
+         goto pv_read_all_pv_of_vg_end;
+      }
 
       if ( ( pv_this = malloc ( ( np + 1) * sizeof ( pv_t*))) == NULL) {
          fprintf ( stderr, "malloc error in %s [line %d]\n",
                            __FILE__, __LINE__);
-         return -LVM_EPV_READ_ALL_PV_OF_VG_MALLOC;
+         ret = -LVM_EPV_READ_ALL_PV_OF_VG_MALLOC;
+         goto pv_read_all_pv_of_vg_end;
       }
       memset ( pv_this, 0, ( np + 1) * sizeof ( pv_t*));
    
@@ -115,8 +135,9 @@ int pv_read_all_pv_of_vg ( char *vg_name, pv_t ***pv, int reread) {
 
    if ( ret == 0) *pv = pv_this;
 
+pv_read_all_pv_of_vg_end:
 #ifdef DEBUG
-   debug ( "pv_read_all_pv_of_vg -- LEAVING with ret: %d\n", ret);
+   debug_leave ( "pv_read_all_pv_of_vg -- LEAVING with ret: %d\n", ret);
 #endif
    return ret;
-}
+} /* pv_read_all_pv_of_vg() */
