@@ -1,10 +1,11 @@
 /*
  * tools/lib/pv_write.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Germany
  *
  * March 1997
  * August-September 1998
+ * January 2000
  *
  *
  * This LVM library is free software; you can redistribute it and/or
@@ -29,6 +30,7 @@
  *
  *    30/08/1998 - seperated disk and core pv structure in pv_write()
  *                 by using pv_copy_to_disk()
+ *    08/02/2000 - use debug_enter()/debug_leave()
  *
  */
 
@@ -41,34 +43,33 @@ int pv_write ( char *pv_name, pv_t *pv) {
    pv_disk_t *pv_disk = NULL;
 
 #ifdef DEBUG
-   debug ( "pv_write -- CALLED with %s %X\n", pv_name, (uint) pv);
+   debug_enter ( "pv_write -- CALLED with %s %X\n", pv_name, (uint) pv);
 #endif
 
    if ( pv_name == NULL ||
         pv_check_name ( pv_name) < 0 ||
         pv == NULL ||
-        pv_check_consistency ( pv) < 0) return -LVM_EPARAM;
-
-   /* covert core to disk data */
-   pv_disk = pv_copy_to_disk ( pv);
-
-   if ( ( pv_handle = open ( pv_name, O_WRONLY)) == -1)
-      ret = -LVM_EPV_WRITE_OPEN;
-   else if ( lseek ( pv_handle, pv->pv_on_disk.base, SEEK_SET) !=
-             pv->pv_on_disk.base) ret = -LVM_EPV_WRITE_LSEEK;
-   else if ( write ( pv_handle, pv_disk,
-                     sizeof ( pv_disk_t)) != sizeof ( pv_disk_t))
-      ret = -LVM_EPV_WRITE_WRITE;
-
-   free ( pv_disk);
-
-   if ( pv_handle != -1) {
-      fsync ( pv_handle);
-      close ( pv_handle);
+        pv_check_consistency ( pv) < 0) ret = -LVM_EPARAM;
+   else {
+      /* covert core to disk data */
+      pv_disk = pv_copy_to_disk ( pv);
+      if ( ( pv_handle = open ( pv_name, O_WRONLY)) == -1)
+         ret = -LVM_EPV_WRITE_OPEN;
+      else if ( lseek ( pv_handle, pv->pv_on_disk.base, SEEK_SET) !=
+                pv->pv_on_disk.base) ret = -LVM_EPV_WRITE_LSEEK;
+      else if ( write ( pv_handle, pv_disk,
+                        sizeof ( pv_disk_t)) != sizeof ( pv_disk_t))
+         ret = -LVM_EPV_WRITE_WRITE;
+      free ( pv_disk);
+   
+      if ( pv_handle != -1) {
+         fsync ( pv_handle);
+         close ( pv_handle);
+      }
    }
 
 #ifdef DEBUG
-   debug ( "pv_write -- LEAVING\n");
+   debug_leave ( "pv_write -- LEAVING\n");
 #endif
    return ret;
 }
