@@ -1,10 +1,11 @@
 /*
  * tools/lib/pv_write_namelist.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Germany
  *
  * March 1997
  * August-September 1998
+ * January 2000
  *
  *
  * This LVM library is free software; you can redistribute it and/or
@@ -24,6 +25,13 @@
  *
  */
 
+/*
+ * Changelog
+ *
+ *    08/02/2000 - use debug_enter()/debug_leave()
+ *
+ */
+  
 #include <liblvm.h>
 
 
@@ -35,23 +43,22 @@ int pv_write_namelist ( char *pv_name, vg_t *vg) {
    char *pv_name_list = NULL;
 
 #ifdef DEBUG
-   debug ( "pv_write_namelist -- CALLED\n");
+   debug_enter ( "pv_write_namelist -- CALLED\n");
 #endif
 
    if ( pv_name == NULL || pv_check_name ( pv_name) < 0 ||
-        vg == NULL || vg_check_name ( vg->vg_name) < 0) return -LVM_EPARAM;
-
-   if ( ( pv_handle = open ( pv_name, O_WRONLY)) == -1)
-      ret = -LVM_EPV_STORE_NAMELIST_OPEN;
+        vg == NULL || vg_check_name ( vg->vg_name) < 0) ret = -LVM_EPARAM;
+   else if ( ( pv_handle = open ( pv_name, O_WRONLY)) == -1)
+      ret = -LVM_EPV_WRITE_NAMELIST_OPEN;
    else if ( lseek ( pv_handle, vg->pv[0]->pv_namelist_on_disk.base,
                      SEEK_SET) != vg->pv[0]->pv_namelist_on_disk.base)
-      ret = -LVM_EPV_STORE_NAMELIST_LSEEK;
+      ret = -LVM_EPV_WRITE_NAMELIST_LSEEK;
    else {
       size = vg->pv_cur * NAME_LEN;
       if ( ( pv_name_list = malloc ( size)) == NULL) {
          fprintf ( stderr, "malloc error in %s [line %d]\n",
                            __FILE__, __LINE__);
-         ret = LVM_EPV_STORE_NAMELIST_MALLOC;
+         ret = LVM_EPV_WRITE_NAMELIST_MALLOC;
       } else {
          memset ( pv_name_list, 0, size);
          for ( p = 0; p < vg->pv_cur; p++) {
@@ -59,7 +66,7 @@ int pv_write_namelist ( char *pv_name, vg_t *vg) {
                strcpy ( &pv_name_list[p*NAME_LEN], vg->pv[p]->pv_name);
          }
          if ( write ( pv_handle, pv_name_list, size) != size)
-            ret = -LVM_EPV_STORE_NAMELIST_WRITE;
+            ret = -LVM_EPV_WRITE_NAMELIST_WRITE;
          free ( pv_name_list);
       }
    }
@@ -70,7 +77,7 @@ int pv_write_namelist ( char *pv_name, vg_t *vg) {
    }
 
 #ifdef DEBUG
-   debug ( "pv_write_namelist -- LEAVING\n");
+   debug_leave ( "pv_write_namelist -- LEAVING with ret: %d\n", ret);
 #endif
    return ret;
 }
