@@ -1,10 +1,11 @@
 /*
  * tools/lib/vg_check_exist.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Germany
  *
  * March, November 1997
- * February,May 1999
+ * February,May,September 1999
+ * January 2000
  *
  *
  * This LVM library is free software; you can redistribute it and/or
@@ -31,6 +32,8 @@
  *                 structure version
  *    17/02/1999 - changed to find any vg_name
  *    20/05/1999 - fixed NULL pointer bug
+ *    29/10/1999 - fixed possible free() bug
+ *    08/02/2000 - use debug_enter()/debug_leave()
  *
  */
 
@@ -39,44 +42,48 @@
 int vg_check_exist ( char *vg_name) {
    int pv_count = 0;
    int p = 0;
-   int ret = 0;
+   int ret = UNDEF;
    vg_t *vg = NULL;
    pv_t **pv = NULL;
 
 #ifdef DEBUG
-   debug ( "vg_check_exist -- CALLED with %s\n", vg_name);
+   debug_enter ( "vg_check_exist -- CALLED with %s\n", vg_name);
 #endif
 
-   if ( vg_name == NULL || vg_check_name ( vg_name) < 0) return -LVM_EPARAM;
-
-   if ( ( ret = vg_read ( vg_name, &vg)) < 0) return ret;
+   if ( vg_name == NULL || vg_check_name ( vg_name) < 0) ret = -LVM_EPARAM;
+   else if ( ( ret = vg_read ( vg_name, &vg)) == 0) {
 #ifdef DEBUG
-   debug ( "vg_check_exist -- before pv_read_all_pv_of_vg\n");
+      debug ( "vg_check_exist -- before pv_read_all_pv_of_vg\n");
 #endif
-   if ( ( ret = pv_read_all_pv_of_vg ( vg_name, &pv, TRUE)) < 0) return ret;
-
-   for ( p = 0; pv != NULL && pv[p] != NULL; p++) {
-      if ( system_id_check_exported ( pv[p]->system_id) == TRUE)
-         pv[p]->vg_name[strlen(pv[p]->vg_name)-strlen(EXPORTED)] = 0;
-      if ( strcmp ( vg_name, pv[p]->vg_name) == 0) {
-         pv_count++;
-         if ( pv[p]->version != LVM_STRUCT_VERSION)
-            return -LVM_EVG_READ_LVM_STRUCT_VERSION;
-         if ( system_id_check_exported ( pv[p]->system_id) == TRUE)
-            return -LVM_EPV_READ_PV_EXPORTED;
+      if ( ( ret = pv_read_all_pv_of_vg ( vg_name, &pv, TRUE)) == 0) {
+         ret = TRUE;
+         for ( p = 0; pv != NULL && pv[p] != NULL; p++) {
+            if ( system_id_check_exported ( pv[p]->system_id) == TRUE)
+               pv[p]->vg_name[strlen(pv[p]->vg_name)-strlen(EXPORTED)] = 0;
+            if ( strcmp ( vg_name, pv[p]->vg_name) == 0) {
+               pv_count++;
+               if ( pv[p]->version != LVM_STRUCT_VERSION) {
+                  ret = -LVM_EVG_READ_LVM_STRUCT_VERSION;
+                  break;
+               }
+               if ( system_id_check_exported ( pv[p]->system_id) == TRUE) {
+                  ret = -LVM_EPV_READ_PV_EXPORTED;
+                  break;
+               }
+            }
+         }
       }
+#ifdef DEBUG
+      debug ( "vg_check_exist -- before vg->pv_cur check with "
+               "vg->pv_cur: %lu  pv_count: %d\n", vg->pv_cur, pv_count);
+#endif
+      if ( vg->pv_cur != pv_count) ret = -LVM_EVG_CHECK_EXIST_PV_COUNT;
    }
 
 #ifdef DEBUG
-   debug ( "vg_check_exist -- before vg->pv_cur check with "
-            "vg->pv_cur: %lu  pv_count: %d\n", vg->pv_cur, pv_count);
+   debug_leave ( "vg_check_exist -- LEAVING with ret: %d\n", ret);
 #endif
-   if ( vg->pv_cur != pv_count) return -LVM_EVG_CHECK_EXIST_PV_COUNT;
-
-#ifdef DEBUG
-   debug ( "vg_check_exist -- LEAVING\n");
-#endif
-   return TRUE;
+   return ret;
 }
 
 
@@ -90,7 +97,7 @@ char **vg_check_exist_all_vg ( void) {
    pv_t **pv = NULL;
 
 #ifdef DEBUG
-   debug ( "vg_check_exist_all_vg -- CALLED\n");
+   debug_enter ( "vg_check_exist_all_vg -- CALLED\n");
 #endif
 
    if ( vg_name_ptr != NULL) {
@@ -99,7 +106,10 @@ char **vg_check_exist_all_vg ( void) {
       vg_name_ptr = NULL;
    }
 
-   if ( ( ret = pv_read_all_pv ( &pv, TRUE)) < 0 || pv == NULL ) return NULL;
+   if ( ( ret = pv_read_all_pv ( &pv, TRUE)) < 0 || pv == NULL ) {
+      vg_name_ptr = NULL;
+      goto vg_check_exist_all_vg_end;
+   }
 
    nv = 0;
    for ( p = 0; pv[p] != NULL; p++) {
@@ -130,11 +140,10 @@ char **vg_check_exist_all_vg ( void) {
                if ( vg_name_ptr_sav != NULL) {
                   for ( v = 0; vg_name_ptr_sav[v] != NULL; v++)
                      free ( vg_name_ptr_sav[v]);
-                  free ( vg_name_ptr);
-                  vg_name_ptr = NULL;
+                  free ( vg_name_ptr_sav);
                }
-               return NULL;
-            }
+               goto vg_check_exist_all_vg_end;
+            } else vg_name_ptr_sav = NULL;
             vg_name_ptr[nv+1] = NULL;
             if ( ( vg_name_ptr[nv] =
                    malloc ( strlen ( pv[p]->vg_name) + 1)) == NULL) {
@@ -144,7 +153,7 @@ char **vg_check_exist_all_vg ( void) {
                   free ( vg_name_ptr[v]);
                free ( vg_name_ptr);
                vg_name_ptr = NULL;
-               return NULL;
+               goto vg_check_exist_all_vg_end;
             }
             strcpy ( vg_name_ptr[nv], pv[p]->vg_name);
 #ifdef DEBUG
@@ -156,9 +165,9 @@ char **vg_check_exist_all_vg ( void) {
       }
    }
 
+vg_check_exist_all_vg_end:
 #ifdef DEBUG
-   debug ( "vg_check_exist_all_vg -- LEAVING with %s\n", vg_name_ptr);
+   debug_leave ( "vg_check_exist_all_vg -- LEAVING with %s\n", vg_name_ptr);
 #endif
-
    return vg_name_ptr;
 }
