@@ -1,9 +1,10 @@
 /*
  * tools/lib/vg_deactivate.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Germany
  *
  * June 1998
+ * January,February 2000
  *
  *
  * This LVM library is free software; you can redistribute it and/or
@@ -23,19 +24,34 @@
  *
  */
 
+/*
+ * Changelog
+ *
+ *    08/02/2000 - use debug_enter()/debug_leave()
+ *    15/02/2000 - use lvm_error()
+ *
+ */
+  
 #include <liblvm.h>
 
 void vg_deactivate ( char *vg_name) {
    int ret = 0;
 
+#ifdef DEBUG
+   debug_enter ( "vg_deactivate -- CALLED\n");
+#endif
+
    if ( vg_check_active ( vg_name) == TRUE) {
       printf ( "%s -- trying to deactivate inconsistent "
                "volume group %s\n", cmd, vg_name);
       if ( ( ret = vg_remove ( vg_name)) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d trying to deactivate %s\n",
-                             cmd, ret, vg_name);
+         fprintf ( stderr, "%s -- ERROR \"%s\" trying to deactivate %s\n",
+                             cmd, lvm_error ( ret), vg_name);
       } else printf ( "%s -- volume group %s deactivated\n", cmd, vg_name);
    }
 
+#ifdef DEBUG
+   debug_leave ( "vg_deactivate -- LEAVING\n");
+#endif
    return;
 }
