@@ -5,6 +5,7 @@
  *
  * November 1997
  * January 1999
+ * February 2000
  *
  *
  * This LVM library is free software; you can redistribute it and/or
@@ -24,24 +25,34 @@
  *
  */
 
+/*
+ * Changelog
+ *
+ *    08/02/2000 - use debug_enter()/debug_leave()
+ *    12/02/2000 - use lvm_remove_recursive() instead of system ( "rm -fr ...
+ *
+ */
+  
 #include <liblvm.h>
 
 int vg_remove_dir_and_group_and_nodes ( char *vg_name) {
-   char system_buffer[NAME_LEN+20];
+   int ret = 0;
+   char buffer[NAME_LEN+20];
 
 #ifdef DEBUG
-   debug ( "vg_remove_dir_and_group_and_nodes -- CALLED\n");
+   debug_enter ( "vg_remove_dir_and_group_and_nodes -- CALLED\n");
 #endif
 
-   if ( vg_name == NULL || vg_check_name ( vg_name) < 0) return -LVM_EPARAM;
-
-   /* remove the volume group directory */
-   sprintf ( system_buffer, "rm -fr " LVM_DIR_PREFIX "%s%c", vg_name, 0);
+   if ( vg_name != NULL && vg_check_name ( vg_name) == 0) {
+      /* remove the volume group directory */
+      sprintf ( buffer, LVM_DIR_PREFIX "%s%c", vg_name, 0);
+      ret = lvm_remove_recursive ( buffer);
+   } else ret = -LVM_EPARAM;
 
 #ifdef DEBUG
-   debug ( "vg_remove_dir_and_group_and_nodes -- LEAVING\n");
+   debug_leave ( "vg_remove_dir_and_group_and_nodes -- LEAVING with ret: %d\n",
+                 ret);
 #endif
-
-   return system ( system_buffer);
+   return ret;
 }
    
