@@ -1,11 +1,12 @@
 /*
  * tools/lvchange.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Germany
  *
  * May,October 1997
  * April-June,August 1998
- * February 1999
+ * February,July,August,October 1999
+ * February 2000
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,6 +36,11 @@
  *    27/06/1998 - changed lvm_tab_* calling convention
  *    04/09/1998 - fixed lv_write_all_pv() called every time
  *    09/02/1999 - implemented change of read ahead sectors
+ *    04/08/1999 - support for snapshot logical volumes
+ *    15/08/1999 - enhanced to avoid attribute changes on
+ *                 snapshot logical volumes
+ *    06/10/1999 - implemented support for long options
+ *    15/02/2000 - use lvm_error()
  *
  */
 
@@ -58,19 +64,33 @@ int main ( int argc, char **argv)
    int lv_allocation = 0; /* bitfield */
    int lv_stat = 0; /* bitfield */
    int opt_A = 1;
+   int opt_A_set = 0;
    int opt_a = 0;
    int opt_C = 0;
    int opt_p = 0;
    int opt_r = 0;
    int opt_v = 0;
-   int read_ahead = 0;
+   long read_ahead = 0;
    int ret = 0;
    char *lv_name = NULL;
 #ifdef DEBUG
-   char *options = "A:a:h?C:dp:r:v";
+   char *options = "A:a:C:dh?p:r:v";
 #else
-   char *options = "A:a:h?C:p:r:v";
+   char *options = "A:a:C:h?p:r:v";
 #endif
+   struct option long_options[] = {
+      { "autobackup", required_argument, NULL, 'A'},
+      { "available",  required_argument, NULL, 'a'},
+      { "contiguous", required_argument, NULL, 'C'},
+#ifdef DEBUG
+      { "debug",      no_argument,       NULL, 'd'},
+#endif
+      { "help",       no_argument,       NULL, 'h'},
+      { "permission", required_argument, NULL, 'p'},
+      { "readahead",  required_argument, NULL, 'r'},
+      { "verbose",    no_argument,       NULL, 'v'},
+      { NULL, 0, NULL, 0}
+   };
    char *vg_name = NULL;
    lv_t *lv = NULL;
    vg_t *vg = NULL;
@@ -80,9 +100,11 @@ int main ( int argc, char **argv)
    SUSER_CHECK;
    LVMTAB_CHECK;
 
-   while ( ( c = getopt ( argc, argv, options)) != EOF) {
+   while ( ( c = getopt_long ( argc, argv, options,
+                               long_options, NULL)) != EOF) {
       switch ( c) {
          case 'A':
+	    opt_A_set++;
             if ( opt_A > 1) {
                fprintf ( stderr, "%s -- A option already given\n\n", cmd);
                return LVM_EINVALID_CMD_LINE;
@@ -135,22 +157,22 @@ int main ( int argc, char **argv)
 
          case 'h':
          case '?':
-            printf ( "\n%s\n\n%s -- Logical Volume Change\n\n"
+            printf ( "\n%s (IOP %d)\n\n%s -- Logical Volume Change\n\n"
                      "Synopsis:\n"
                      "---------\n\n"
                      "%s\n"
-                     "\t[-A y/n]\n"
-                     "\t[-a y/n]\n"
-                     "\t[-C y/n]\n"
+                     "\t[-A/--autobackup y/n]\n"
+                     "\t[-a/--available y/n]\n"
+                     "\t[-C/--contiguous y/n]\n"
 #ifdef DEBUG
-                     "\t[-d]\n"
+                     "\t[-d/--debug]\n"
 #endif
-                     "\t[-h/-?]\n"
-                     "\t[-p r/rw]\n"
-                     "\t[-r ReadAheadSectors]\n"
-                     "\t[-v]\n"
+                     "\t[-h/-?/--help]\n"
+                     "\t[-p/--permission r/rw]\n"
+                     "\t[-r/--readahead ReadAheadSectors]\n"
+                     "\t[-v/--verbose]\n"
                      "\tLogicalVolumePath [LogicalVolumePath...]\n\n",
-                     lvm_version, cmd, cmd);
+                     lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd);
             return 0;
             break;
 
@@ -199,6 +221,8 @@ int main ( int argc, char **argv)
    }
   
    CMD_MINUS_CHK;
+   CMD_CHECK_OPT_A_SET;
+
 
    if ( opt_a == 0 && opt_C == 0 && opt_p == 0 && opt_r == 0) {
       fprintf ( stderr, "%s -- please give one or more options a, C, p "
@@ -231,8 +255,12 @@ int main ( int argc, char **argv)
       /* does VG exist? */
       if ( opt_v > 0) printf ( "%s -- checking volume group existence\n", cmd);
       if ( lvm_tab_vg_check_exist ( vg_name, NULL) != TRUE) {
-         fprintf ( stderr, "%s -- can't change logical volume: volume group "
-                           "\"%s\" doesn't exist\n\n", cmd, vg_name);
+         if(vg_name)
+            fprintf ( stderr, "%s -- can't change logical volume: volume group "
+                      "\"%s\" doesn't exist\n\n", cmd, vg_name);
+         else
+            fprintf ( stderr, "%s -- can't change logical volume: need to "
+                      "specify volume group\n\n", cmd);
          return LVM_ELVCHANGE_VG_CHECK_EXIST;
       }
 
@@ -252,8 +280,8 @@ int main ( int argc, char **argv)
             fprintf ( stderr, "%s -- volume group \"%s\" is exported\n\n",
                       cmd, vg_name);
          } else {
-            fprintf ( stderr, "%s -- ERROR %d: reading data of \"%s\"\n\n",
-                      cmd, ret, vg_name);
+            fprintf ( stderr, "%s -- ERROR \"%s\" reading data of \"%s\"\n\n",
+                      cmd, lvm_error ( ret), vg_name);
          }
          continue;
       }
@@ -270,8 +298,9 @@ int main ( int argc, char **argv)
       if ( opt_v > 0) printf ( "%s -- checking consistency of \"%s\"\n",
                                cmd, vg_name);
       if ( ( ret = vg_check_consistency_with_pv_and_lv ( vg)) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d checking consistency of \"%s\"\n\n",
-                   cmd, ret, vg_name);
+         fprintf ( stderr, "%s -- ERROR \"%s\" checking consistency of "
+                           "\"%s\"\n\n",
+                           cmd, lvm_error ( ret), vg_name);
          continue;
       }
       
@@ -288,20 +317,33 @@ int main ( int argc, char **argv)
       if ( opt_v > 0) printf ( "%s -- getting status of \"%s\" from VGDA in "
                                "kernel\n", cmd, lv_name);
       if ( ( ret = lv_status_byname ( vg_name, lv_name, &lv)) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d getting status of logical "
-                           "volume \"%s\"\n\n", cmd, ret, lv_name);
+         fprintf ( stderr, "%s -- ERROR \"%s\" getting status of logical "
+                           "volume \"%s\"\n\n",
+                           cmd, lvm_error ( ret), lv_name);
          continue;
       }
    
+      if ( lv->lv_access & LV_SNAPSHOT) {
+         fprintf ( stderr, "%s -- change on snapshot logical "
+                           "volume \"%s\" not allowed\n", cmd, lv_name);
+         continue;
+      }
+
+      if ( lv->lv_access & LV_SNAPSHOT_ORG) {
+         fprintf ( stderr, "%s -- change on logical volume \"%s\" under "
+                           "snapshot not allowed\n", cmd, lv_name);
+         continue;
+      }
+
       lvm_dont_interrupt ( 0);
    
       if ( opt_v > 0) printf ( "%s -- changing logical volume \"%s\" in VGDA "
                                "of kernel\n", cmd, lv_name);
       /* O_RDONLY to avoid error when LV is inactive!!! (see lvm_blk_open) */
       if ( ( blkdev = open ( lv_name, O_RDONLY)) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d: couldn't open logical "
+         fprintf ( stderr, "%s -- ERROR \"%s\" couldn't open logical "
                            "volume \"%s\"\n\n",
-                           cmd, errno, lv_name);
+                           cmd, lvm_error ( errno), lv_name);
          return LVM_ELVCHANGE_LV_OPEN;
       }
    
@@ -333,9 +375,9 @@ int main ( int argc, char **argv)
                                     "for (open) logical volume \"%s\"\n\n",
                                     cmd, lv_name);
                } else {
-                  fprintf ( stderr, "%s -- ERROR %d: couldn't set access "
+                  fprintf ( stderr, "%s -- ERROR \"%s\" couldn't set access "
                                     "permissions for logical volume \"%s\"\n\n",
-                                    cmd, ret, lv_name);
+                                    cmd, lvm_error ( ret), lv_name);
                }
                return LVM_ELVCHANGE_LV_SET_ACCESS;
             }
@@ -375,9 +417,9 @@ int main ( int argc, char **argv)
                                     "for (open) logical volume \"%s\"\n\n",
                                     cmd, lv_name);
                } else {
-                  fprintf ( stderr, "%s -- ERROR %d: couldn't set state for "
+                  fprintf ( stderr, "%s -- ERROR \"%s\" couldn't set state for "
                                     "logical volume \"%s\"\n\n",
-                                    cmd, ret, lv_name);
+                                    cmd, lvm_error ( ret), lv_name);
                }
                return LVM_ELVCHANGE_LV_SET_STATUS;
             }
@@ -425,10 +467,10 @@ int main ( int argc, char **argv)
                                        "for (open) logical volume \"%s\"\n\n",
                                        cmd, lv_name);
                   } else {
-                     fprintf ( stderr, "%s -- ERROR %d: couldn't set "
+                     fprintf ( stderr, "%s -- ERROR \"%s\" couldn't set "
                                        "allocation for logical volume "
                                        "\"%s\"\n\n",
-                                       cmd, ret, lv_name);
+                                       cmd, lvm_error ( ret), lv_name);
                   }
                   return LVM_ELVCHANGE_LV_SET_ALLOCATION;
                }
@@ -449,13 +491,13 @@ int main ( int argc, char **argv)
             return LVM_ELVCHANGE_READ_AHEAD;
          }
          if ( vg->lv[l]->lv_read_ahead == read_ahead) {
-            fprintf ( stderr, "%s -- read ahead sector already is %d\n\n",
+            fprintf ( stderr, "%s -- read ahead sector already is %ld\n\n",
                               cmd, read_ahead);
             return LVM_ELVCHANGE_READ_AHEAD;
          }
          if ( ( ret = lv_change_read_ahead ( lv_name, read_ahead)) < 0) {
-            fprintf ( stderr, "%s -- ERROR %d read ahead sector change\n\n",
-                              cmd, ret);
+            fprintf ( stderr, "%s -- ERROR \"%s\" read ahead sector change\n\n",
+                              cmd, lvm_error ( ret));
             return LVM_ELVCHANGE_READ_AHEAD;
          }
          vg->lv[l]->lv_read_ahead = read_ahead;
@@ -470,9 +512,9 @@ int main ( int argc, char **argv)
          if ( opt_v > 0) printf ( "%s -- storing logical volume on disk(s)\n",
                                   cmd);
          if ( ( ret = lv_write_all_pv ( vg, l)) < 0) {
-            fprintf ( stderr, "%s -- ERROR %d: couldn't store logical volume "
-                              "information of \"%s\"\n",
-                      cmd, ret, lv_name);
+            fprintf ( stderr, "%s -- ERROR \"%s\" couldn't store logical "
+                              "volume information of \"%s\"\n",
+                              cmd, lvm_error ( ret), lv_name);
             return LVM_ELVCHANGE_LV_WRITE_ALL_PV;
          }
    
@@ -484,6 +526,10 @@ int main ( int argc, char **argv)
             printf ( "%s -- doing automatic backup of volume group \"%s\"\n",
                      cmd, vg_name);
             vg_cfgbackup ( vg_name, VG_BACKUP_DIR, opt_v, vg);
+         } else {
+            printf ( "%s -- WARNING: you don't have an automatic "
+                     "backup of \"%s\"\n",
+                     cmd, vg_name);
          }
       } else  printf ( "%s -- logical volume \"%s\" didn't have "
                        "to be changed\n", cmd, lv_name);
