@@ -1,11 +1,12 @@
 /*
  * tools/lvcreate.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Germany
  *
  * March,October,November 1997
  * January,May-July,September,December 1998
- * January,February 1999
+ * January,February,July,October 1999
+ * February 2000
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -50,6 +51,11 @@
  *    28/01/1999 - fixed logical volume path too long in command line
  *    06/02/1999 - fixed lvm_check_number() usage
  *    16/02/1999 - changed to new lv_create_node() calling convention
+ *    26/07/1999 - support snapshot logical volume creation
+ *    06/10/1999 - implemented support for long options
+ *    16/11/1999 - checked for option -l or -L given on command line
+ *    15/02/2000 - use lvm_error()
+ *    16/02/2000 - use new function lv_setup_snapshot_exception_table()
  *
  */
 
@@ -72,11 +78,14 @@ int opt_d = 0;
 
 int main ( int argc, char **argv) {
    int c = 0;
+   int chunk_size = LVM_SNAPSHOT_DEF_CHUNK;
    int i = 0;
    int l = 0;
    int lv_handle = -1;
    int pa = 0;
    int opt_A = 1;
+   int opt_A_set = 0;
+   int opt_c = 0;
    int opt_C = 0;
    int opt_i = 0;
    int opt_I = 0;
@@ -85,12 +94,13 @@ int main ( int argc, char **argv) {
    int opt_n = 0;
    int opt_p = 0;
    int opt_r = 0;
+   int opt_s = 0;
    int opt_v = 0;
    int opt_Z = 1;
    int ret = 0;
    uint size = 0;
    int size_rest = 0;
-   int read_ahead = 0;
+   long read_ahead = 0;
    uint allocation = 0;
    uint mirrors = 0;
    uint permission = LV_READ | LV_WRITE;
@@ -99,17 +109,42 @@ int main ( int argc, char **argv) {
    ulong pe_free = 0;
    char buf[2*BLOCK_SIZE];
 #ifdef DEBUG
-   char *options = "A:C:dh?i:I:l:L:n:p:r:vZ:";
+   char *options = "A:c:C:dh?i:I:l:L:n:p:r:svZ:";
 #else
-   char *options = "A:C:h?i:I:l:L:n:p:r:vZ:";
+   char *options = "A:c:C:h?i:I:l:L:n:p:r:svZ:";
 #endif
+   struct option long_options[] = {
+      { "autobackup",  required_argument, NULL, 'A'},
+      { "chunksize",   required_argument, NULL, 'c'},
+      { "contiguous",  required_argument, NULL, 'C'},
+#ifdef DEBUG
+      { "debug",       no_argument,       NULL, 'd'},
+#endif
+      { "help",        no_argument,       NULL, 'h'},
+      { "stripes",     required_argument, NULL, 'i'},
+      { "stripesize",  required_argument, NULL, 'I'},
+      { "extents",     required_argument, NULL, 'l'},
+      { "size",        required_argument, NULL, 'L'},
+      { "name",        required_argument, NULL, 'n'},
+      { "permissions", required_argument, NULL, 'p'},
+      { "readahead",   required_argument, NULL, 'r'},
+      { "snapshot",    no_argument,       NULL, 's'},
+      { "verbose",     no_argument,       NULL, 'v'},
+      { "zero",        required_argument, NULL, 'Z'},
+      { NULL,          0,                 NULL, 0},
+   };
    char *dummy = NULL;
+   char *dummy1 = NULL;
+   char *lv_name_ptr = NULL;
    char *vg_name = NULL;
    char **pv_allowed = NULL;
    char lv_name[NAME_LEN] = { 0, };
+   char lv_snapshot_name[NAME_LEN] = { 0, };
    char lv_path[NAME_LEN] = { 0, };
    pv_t *pv = NULL;
    vg_t *vg = NULL;
+   vg_t *vg_core = NULL;
+   lv_t *lv = NULL;
    struct stat stat_buf;
 
    cmd = basename ( argv[0]);
@@ -117,9 +152,11 @@ int main ( int argc, char **argv) {
    SUSER_CHECK;
    LVMTAB_CHECK;
 
-   while ( ( c = getopt ( argc, argv, options)) != EOF) {
+   while ( ( c = getopt_long ( argc, argv, options,
+                               long_options, NULL)) != EOF) {
       switch ( c) {
          case 'A':
+            opt_A_set++;
             if ( opt_A > 1) {
                fprintf ( stderr, "%s -- A option already given\n\n", cmd);
                return LVM_EINVALID_CMD_LINE;
@@ -133,6 +170,33 @@ int main ( int argc, char **argv) {
             }
             break;
 
+         case 'c':
+            if ( opt_c > 0) {
+               fprintf ( stderr, "%s -- c option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
+            }
+
+            if ( ( chunk_size = lvm_check_number ( optarg, TRUE)) > 0) {
+               if ( chunk_size < LVM_SNAPSHOT_MIN_CHUNK || 
+                    chunk_size > LVM_SNAPSHOT_MAX_CHUNK || 
+                    ( chunk_size & ( chunk_size - 1)) != 0)  {
+                  dummy  = lvm_show_size ( LVM_SNAPSHOT_MIN_CHUNK, SHORT);
+                  dummy1 = lvm_show_size ( LVM_SNAPSHOT_MAX_CHUNK, SHORT);
+                  fprintf ( stderr, "%s -- ERROR: chunk size must be >= %s "
+                                    "and <= %s and power of 2\n\n",
+                                    cmd, dummy, dummy1);
+                  free ( dummy);
+                  free ( dummy1);
+                  return LVM_EINVALID_CMD_LINE;
+               }
+               opt_c++;
+            } else {
+               fprintf ( stderr, "%s -- ERROR option c argument \"%s\"\n\n",
+                                 cmd, optarg);
+               return LVM_EINVALID_CMD_LINE;
+            }
+            break;
+
          case 'C':
             if ( opt_C > 0) {
                fprintf ( stderr, "%s -- C option already given\n\n", cmd);
@@ -159,26 +223,32 @@ int main ( int argc, char **argv) {
 
          case 'h':
          case '?':
-            printf ( "\n%s\n\n%s -- Logical Volume Create\n\n"
+            printf ( "\n%s (IOP %d)\n\n%s -- Logical Volume Create\n\n"
                      "Synopsis:\n"
                      "---------\n\n"
                      "%s\n"
-                     "\t[-A y/n]\n"
-                     "\t[-C y/n]\n"
+                     "\t[-A/--autobackup y/n]\n"
+                     "\t[-C/--contiguous y/n]\n"
 #ifdef DEBUG
-                     "\t[-d]\n"
+                     "\t[-d/--debug]\n"
 #endif
-                     "\t[-h/?]\n"
-                     "\t[-i Stripes [-I StripeSize]]\n"
-                     "\t{-l LogicalExtentsNumber |\n"
-                     "\t -L LogicalVolumeSize[kKmMgGtT]}\n"
-                     "\t[-n LogicalVolumeName]\n"
-                     "\t[-p r/rw]\n"
-                     "\t[-r ReadAheadSectors]\n"
-                     "\t[-v]\n"
-                     "\t[-Z y/n]\n"
-                     "\tVolumeGroupName [PhysicalVolumePath...]\n\n",
-                     lvm_version, cmd, cmd);
+                     "\t[-h/-?/--help]\n"
+                     "\t[-i/--stripes Stripes [-I/--stripesize StripeSize]]\n"
+                     "\t{-l/--extents LogicalExtentsNumber |\n"
+                     "\t -L/--size LogicalVolumeSize[kKmMgGtT]}\n"
+                     "\t[-n/--name LogicalVolumeName]\n"
+                     "\t[-p/--permission r/rw]\n"
+                     "\t[-r/--readahead ReadAheadSectors]\n"
+                     "\t[-v/--verbose]\n"
+                     "\t[-Z/--zero y/n]\n"
+                     "\tVolumeGroupName [PhysicalVolumePath...]\n\n"
+                     "%s\n"
+                     "\t{-l/--extents LogicalExtentsNumber |\n"
+                     "\t -L/--size LogicalVolumeSize[kKmMgGtT]}\n"
+                     "\t[-c/--chunksize ChunkSize]\n"
+                     "\t-s/--snapshot -n/--name SnapshotLogicalVolumeName\n"
+                     "\tLogicalVolumePath [PhysicalVolumePath...]\n\n",
+                     lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd, cmd);
             return 0;
             break;
 
@@ -287,6 +357,16 @@ int main ( int argc, char **argv) {
             opt_r++;
             break;
 
+         case 's':
+            if ( opt_s > 0) {
+               fprintf ( stderr, "%s -- s option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
+            }
+            permission = LV_READ;
+            opt_s++;
+            opt_Z = 0;
+            break;
+
          case 'v':
             if ( opt_v > 0) {
                fprintf ( stderr, "%s -- v option already given\n\n", cmd);
@@ -317,14 +397,64 @@ int main ( int argc, char **argv) {
    }
 
    CMD_MINUS_CHK;
+   CMD_CHECK_OPT_A_SET;
+
+   if ( opt_L + opt_l == 0) {
+      fprintf ( stderr, "%s -- please enter either option -l or -L\n\n",
+                        cmd);
+      return LVM_EINVALID_CMD_LINE;
+   }
 
    if ( optind == argc) {
-      fprintf ( stderr, "%s -- please enter a volume group name\n\n", cmd);
+      /* snapshot logical volume */
+      if ( opt_s > 0) {
+         fprintf ( stderr, "%s -- please enter a logical volume name for "
+                           "snapshooting\n\n",
+                           cmd);
+      } else {
+         fprintf ( stderr, "%s -- please enter a volume group name\n\n", cmd);
+      }
       return LVM_ELVCREATE_VG_NAME;
    }
-   vg_name = argv[optind];
+
+   /* snapshot logical volume */
+   if ( opt_s > 0) {
+      /* these options are not allowed for snapshot */
+      if ( opt_C + opt_i + opt_I + opt_n + opt_p + opt_r + opt_Z > 0) {
+         fprintf ( stderr, "%s -- invalid command line\n\n", cmd);
+         return LVM_EINVALID_CMD_LINE;
+      }
+      strncpy ( lv_snapshot_name, lv_name, sizeof ( lv_name) - 1);
+      lv_path[sizeof ( lv_name) - 1] = 0;
+      strncpy ( lv_name, argv[optind], sizeof ( lv_name) - 1);
+      lv_name[sizeof ( lv_name) - 1] = 0;
+      vg_name = vg_name_of_lv ( lv_name);
+      sprintf ( lv_path, LVM_DIR_PREFIX "%s/%s%c",
+                vg_name, lv_snapshot_name, 0);
+
+      if ( opt_v > 0) printf ( "%s -- checking snapshot logical volume "
+                               "name \"%s\"\n",
+                               cmd, lv_path);
+      if ( lv_check_name ( lv_path) < 0) {
+         fprintf ( stderr, "%s -- no valid snapshot logical volume "
+                           "name \"%s\"\n\n",
+                           cmd, lv_path);
+         return LVM_ELVCREATE_LV_NAME;
+      } else if ( lstat ( lv_path, &stat_buf) != -1) {
+         lvm_show_filetype ( stat_buf.st_mode, lv_path);
+         return LVM_ELVCREATE_LV_LSTAT;
+      } else if ( lvm_tab_lv_check_exist ( lv_path) == TRUE) {
+         fprintf ( stderr, "%s -- snapshot logical volume \"%s\" "
+                           "already exists\n\n",
+                           cmd, lv_path);
+         return LVM_ELVCREATE_LV_CHECK_EXIST;
+      }
+   } else {
+      vg_name = argv[optind];
+   }
    optind++;
 
+
    if ( opt_v > 0) printf ( "%s -- checking volume group name \"%s\"\n",
                             cmd, vg_name);
    if ( vg_check_name ( vg_name) < 0) {
@@ -348,7 +478,7 @@ int main ( int argc, char **argv) {
       return LVM_ELVCREATE_VG_CHECK_ACTIVE;
    }
 
-   if ( strlen ( lv_name) > 0) {
+   if ( opt_s == 0 && strlen ( lv_name) > 0) {
       if ( strlen ( LVM_DIR_PREFIX) + strlen ( vg_name) +
            strlen ( lv_name) + 1 > NAME_LEN - 1) {
          fprintf ( stderr, "%s -- the logical volume path is longer than "
@@ -402,21 +532,24 @@ int main ( int argc, char **argv) {
       }
    }
 
-   if ( opt_v > 0) printf ( "%s -- checking stripe count\n", cmd);
-   if ( stripes < 1 || stripes > LVM_MAX_STRIPES) {
-      fprintf ( stderr, "%s -- invalid number of stripes: %d\n"
-                        "%s -- must be between %d and %d\n\n",
-                        cmd, stripes, cmd, 1, LVM_MAX_STRIPES);
-      return LVM_ELVCREATE_STRIPES;
+   if ( opt_s == 0) {
+      if ( opt_v > 0) printf ( "%s -- checking stripe count\n", cmd);
+      if ( stripes < 1 || stripes > LVM_MAX_STRIPES) {
+         fprintf ( stderr, "%s -- invalid number of stripes: %d\n"
+                           "%s -- must be between %d and %d\n\n",
+                           cmd, stripes, cmd, 1, LVM_MAX_STRIPES);
+         return LVM_ELVCREATE_STRIPES;
+      }
+
+      if ( opt_v > 0) printf ( "%s -- checking stripe size\n", cmd);
+      if ( stripesize > 0 && lv_check_stripesize ( stripesize) < 0) {
+         fprintf ( stderr, "%s -- invalid stripesize %d\n\n",
+                   cmd, stripesize);
+         return LVM_ELVCREATE_STRIPE_SIZE;
+      }
+      stripesize *= 2;
    }
 
-   if ( opt_v > 0) printf ( "%s -- checking stripe size\n", cmd);
-   if ( stripesize > 0 && lv_check_stripesize ( stripesize) < 0) {
-      fprintf ( stderr, "%s -- invalid stripesize %d\n\n",
-                cmd, stripesize);
-      return LVM_ELVCREATE_STRIPE_SIZE;
-   }
-   stripesize *= 2;
    if ( stripes == 1) stripesize = 0;
 
    LVM_CHECK_IOP;
@@ -424,47 +557,92 @@ int main ( int argc, char **argv) {
 
    if ( opt_v > 0) printf ( "%s -- getting volume group status from "
                             "VGDA in kernel\n", cmd);
-   if ( vg_status ( vg_name, &vg) < 0) {
-      fprintf ( stderr, "%s -- can't create logical volume"
-                        " in inactive volume group \"%s\"\n\n", cmd, vg_name);
+   if ( ( ret = vg_status_with_pv_and_lv ( vg_name, &vg_core)) != 0) {
+      fprintf ( stderr, "%s -- ERROR \"%s\" getting status of volume group "
+                        "\"%s\" from kernel\n\n",
+                        cmd, lvm_error ( ret), vg_name);
       return LVM_ELVCREATE_VG_STATUS;
    }
 
-   if ( opt_v > 0) printf ( "%s -- checking stripe size against "
-                            "volume group physical extent size\n", cmd);
-   if ( stripesize > vg->pe_size) {
-      fprintf ( stderr, "%s -- setting stripe size %d KB to "
-                        "physical extent size %u KB\n",
-                        cmd, stripesize / 2, vg->pe_size / 2);
-      stripesize = vg->pe_size;
+   if ( opt_s == 0) {
+      if ( opt_v > 0) printf ( "%s -- checking stripe size against "
+                               "volume group physical extent size\n", cmd);
+      if ( stripesize > vg_core->pe_size) {
+         fprintf ( stderr, "%s -- setting stripe size %d KB to "
+                           "physical extent size %u KB\n",
+                           cmd, stripesize / 2, vg_core->pe_size / 2);
+         stripesize = vg_core->pe_size;
+      }
    }
 
    if ( opt_L > 0 || opt_l > 0) {
       if ( opt_L > 0) size *= 2;
-      else            size *= vg->pe_size;
-      if ( size % vg->pe_size > 0) {
-         size += vg->pe_size - size % vg->pe_size;
+      else            size *= vg_core->pe_size;
+      if ( size % vg_core->pe_size > 0) {
+         size += vg_core->pe_size - size % vg_core->pe_size;
          printf ( "%s -- rounding up size to physical extent boundary \"%s\"\n",
                   cmd, ( dummy = lvm_show_size ( size / 2, SHORT)));
          free ( dummy); dummy = NULL;
       }
 
-      size_rest = size % ( stripes * vg->pe_size);
+      size_rest = size % ( stripes * vg_core->pe_size);
       if ( size_rest != 0) {
          printf ( "%s -- rounding %d KB to stripe boundary size ",
                   cmd, size / 2);
-         size = size - size_rest + stripes * vg->pe_size;
-         printf ( "%d KB / %u PE\n", size / 2, size / vg->pe_size);
+         size = size - size_rest + stripes * vg_core->pe_size;
+         printf ( "%d KB / %u PE\n", size / 2, size / vg_core->pe_size);
       }
-   } else {
+      if ( opt_s > 0) {
+         if ( opt_v > 0) printf ( "%s -- getting logical volume status "
+                                  "from kernel\n",
+                                  cmd);
+         if ( ( ret = lv_status_byname ( vg_name, lv_name, &lv)) < 0) {
+            fprintf ( stderr, "%s -- ERROR \"%s\" getting status of logical "
+                              "volume \"%s\"\n\n",
+                              cmd, lvm_error ( ret), lv_name);
+            return LVM_ELVCREATE_LV_STATUS_BYNAME;
+         }
+         if ( lv->lv_access & LV_SNAPSHOT) {
+            fprintf ( stderr, "%s -- ERROR: can't create snapshot of "
+                              "snapshot \"%s\"\n"
+                              "%s -- please use original logical volume "
+                              "name\n\n",
+                              cmd, lv_name, cmd);
+            return LVM_EINVALID_CMD_LINE;
+         }
+         if ( size > lv->lv_size) {
+            fprintf ( stderr, "%s -- ERROR: size of snapshot is larger "
+                              "than that of original logical volume "
+                              "\"%s\"\n\n",
+                              cmd, lv->lv_name);
+            return LVM_EINVALID_CMD_LINE;
+         } else if ( size < lv->lv_size) {
+            dummy = lvm_show_size ( size / 2, SHORT);
+            printf ( "%s -- WARNING: all snapshots will be disabled if "
+                     "more than %s are changed\n", cmd, dummy);
+            free ( dummy); dummy = NULL;
+         
+         }
+      }
+   } else if ( opt_s == 0) {
       fprintf ( stderr, "%s -- please give an l or L option\n\n", cmd);
       return LVM_EINVALID_CMD_LINE;
    }
 
+   if ( opt_v > 0) printf ( "%s -- reading volume group data of \"%s\"\n",
+                            cmd, vg_name);
+   if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name, &vg)) != 0) {
+      fprintf ( stderr, "%s -- ERROR \"%s\" couldn't get volume "
+                        "group data of \"%s\"\n\n",
+                        cmd, lvm_error ( ret), vg_name);
+      return LVM_ELVCREATE_VG_READ;
+   }
+
    if ( opt_v > 0) printf ( "%s -- checking logical volume maximum size\n",
                             cmd);
    if ( size > LVM_LV_SIZE_MAX( vg)) {
-      fprintf ( stderr, "%s -- size %d is larger than maximum VGDA kernel size "
+      fprintf ( stderr, "%s -- size %d is larger than maximum VGDA "
+                        "kernel size "
                         "of %llu\n\n",
                         cmd, size / 2, LVM_LV_SIZE_MAX( vg) / 2);
       return LVM_ELVCREATE_LV_SIZE;
@@ -477,29 +655,30 @@ int main ( int argc, char **argv) {
                         "volume group \"%s\"\n\n",
                         cmd, vg_name);
       return LVM_ELVCREATE_PE_FREE;
-   }
-   else if ( size / vg->pe_size > pe_free) {
+   } else if ( size / vg->pe_size > pe_free) {
       fprintf ( stderr, "%s -- only %lu free physical extent%s in "
                         "volume group \"%s\"\n\n",
                         cmd, pe_free, pe_free > 1 ? "s" : "", vg_name);
       return LVM_ELVCREATE_PE_FREE;
    }
 
-   if ( opt_v > 0) printf ( "%s -- checking stripe count against "
-                            "physical volume count\n", cmd);
-   if ( stripes > vg->pv_cur) {
-      fprintf ( stderr, "%s -- too many stripes for \"%s\" with %u "
-                        "physical volume%s\n\n",
-                        cmd, vg_name, vg->pv_cur, vg->pv_cur > 1 ? "s" : "");
-      return LVM_ELVCREATE_STRIPE_COUNT;
+   if ( opt_s == 0) {
+      if ( opt_v > 0) printf ( "%s -- checking stripe count against "
+                               "physical volume count\n", cmd);
+      if ( stripes > vg->pv_cur) {
+         fprintf ( stderr, "%s -- too many stripes for \"%s\" with %u "
+                           "physical volume%s\n\n",
+                           cmd, vg_name, vg->pv_cur, vg->pv_cur > 1 ? "s" : "");
+         return LVM_ELVCREATE_STRIPE_COUNT;
+      }
    }
 
-   if ( opt_v > 0) printf ( "%s -- reading volume group data of \"%s\"\n",
-                            cmd, vg_name);
-   if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name, &vg)) != 0) {
-      fprintf ( stderr, "%s -- ERROR %d: couldn't get volume "
-                        "group data of \"%s\"\n\n",
-                        cmd, ret, vg_name);
+   if ( opt_s > 0 && ( chunk_size * 2) > vg->pe_size) {
+      dummy = lvm_show_size ( vg->pe_size * 2, SHORT);
+      fprintf ( stderr, "%s -- ERROR: snapshot chunk size must be less "
+                        "than or equal to PE size %s!\n\n",
+                        cmd, dummy);
+      free ( dummy); dummy = NULL;
       return LVM_ELVCREATE_VG_READ;
    }
 
@@ -534,12 +713,15 @@ int main ( int argc, char **argv) {
       return LVM_ELVCREATE_LV_COUNT;
    }
 
+   if ( opt_s > 0) lv_name_ptr = lv_snapshot_name;
+   else            lv_name_ptr = lv_name;
+
    /* setup logical volume */
    if ( opt_v > 0) printf ( "%s -- setting up logical volume\n", cmd);
-   if ( ( ret = lv_setup_for_create ( vg_name, &vg, lv_name, &l, size,
+   if ( ( ret = lv_setup_for_create ( vg_name, &vg, lv_name_ptr, &l, size,
                                       stripes, stripesize, mirrors,
                                       allocation, permission,
-                                      pv_allowed)) != 0) {
+                                      pv_allowed)) < 0) {
       if ( ret == -LVM_ESIZE) {
          if ( pv_allowed != NULL)
             fprintf ( stderr, "%s -- not enough allocatable/free space "
@@ -556,13 +738,12 @@ int main ( int argc, char **argv) {
                            cmd);
          return LVM_ELVCREATE_NO_DEV;
       } else {
-         fprintf ( stderr, "%s -- ERROR %d: creating \"%s\" in \"%s\"\n\n", 
-                   cmd, ret, lv_name, vg_name);
+         fprintf ( stderr, "%s -- ERROR \"%s\" creating \"%s\" in \"%s\"\n\n", 
+                   cmd, lvm_error ( ret), lv_name, vg_name);
          return LVM_ELVCREATE_LV_SETUP;
       }
    }
 
-
    /* read ahead sector change; must be after lv_setup_for_create! */
    if ( opt_v > 0) printf ( "%s -- setting read ahead sectors\n", cmd);
    if ( opt_r > 0) {
@@ -575,16 +756,25 @@ int main ( int argc, char **argv) {
       vg->lv[l]->lv_read_ahead = read_ahead;
    }
    
+   /* setup snapshot logical volume exception structs */
+   if ( opt_s > 0 &&
+        ( ret = lv_setup_snapshot_exception_table ( vg_core, lv_name, vg,
+                                                    l, chunk_size)) < 0) {
+      fprintf ( stderr, "%s -- ERROR \"%s\" setting up snapshot exception "
+                        "table for \"%s\"\n\n",
+                        cmd, lvm_error ( ret), lv_name_ptr);
+      return LVM_ELVCREATE_LV_SETUP_SNAPSHOT_EXCEPTION_TABLE;
+   }
 
    lvm_dont_interrupt ( 0);
 
    /* create it in kernel */
    if ( opt_v > 0) printf ( "%s -- creating logical volume VGDA "
                             "in kernel\n", cmd);
-   if ( ( ret = lv_create ( vg, vg->lv[l], lv_name)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d creating VGDA for \"%s\" "
+   if ( ( ret = lv_create ( vg, vg->lv[l], vg->lv[l]->lv_name)) < 0) {
+      fprintf ( stderr, "%s -- ERROR \"%s\" creating VGDA for \"%s\" "
                         "in kernel\n\n", 
-                        cmd, ret, lv_name);
+                        cmd, lvm_error ( ret), lv_name_ptr);
       return LVM_ELVCREATE_LV_CREATE;
    }
 
@@ -592,12 +782,13 @@ int main ( int argc, char **argv) {
    if ( opt_v > 0) printf ( "%s -- storing logical volume VGDA on disk(s)\n",
                             cmd);
    if ( ( ret = vg_write_with_pv_and_lv ( vg)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d: storing volume group data of \"%s\""
+      fprintf ( stderr, "%s -- ERROR \"%s\" storing volume group data of \"%s\""
                         " on disks\n\n", 
-                cmd, ret, vg_name);
-      if ( ( ret = lv_remove ( vg, vg->lv[l], lv_name)) < 0)
-         fprintf ( stderr, "%s -- ERROR %d removing VGDA for \"%s\" "
-                           "from kernel\n\n", cmd, ret, lv_name);
+                        cmd, lvm_error ( ret), vg_name);
+      if ( ( ret = lv_remove ( vg, vg->lv[l], lv_name_ptr)) < 0)
+         fprintf ( stderr, "%s -- ERROR \"%s\" removing VGDA for \"%s\" "
+                           "from kernel\n\n",
+                           cmd, lvm_error ( ret), lv_name_ptr);
       return LVM_ELVCREATE_VG_WRITE;
    }
 
@@ -606,38 +797,41 @@ int main ( int argc, char **argv) {
       if ( ret == -LVM_ELV_CREATE_MODE_MKNOD) {
          fprintf ( stderr, "%s -- logical volume node \"%s\" already "
                            "existent\n\n",
-                           cmd, lv_name);
+                           cmd, lv_name_ptr);
       } else {
-         fprintf ( stderr, "%s -- ERROR %d creating logical volume "
+         fprintf ( stderr, "%s -- ERROR \"%s\" creating logical volume "
                            "node \"%s\"\n\n",
-                           cmd, ret, lv_name);
+                           cmd, lvm_error ( ret), lv_name_ptr);
       }
       return LVM_ELVCREATE_LV_CREATE_NODE;
    }
 
    if ( opt_Z > 0) {
       if ( opt_v > 0) printf ( "%s -- opening logical volume \"%s\" for init\n",
-                               cmd, lv_name);
-      if ( ( lv_handle = open ( lv_name, O_WRONLY)) == -1) {
-         fprintf ( stderr, "%s -- ERROR %d opening logical volume \"%s\"\n\n",
-                   cmd, errno, lv_name);
+                               cmd, lv_name_ptr);
+      if ( ( lv_handle = open ( lv_name_ptr, O_WRONLY)) == -1) {
+         fprintf ( stderr, "%s -- ERROR \"%s\" opening logical "
+                           "volume \"%s\"\n\n",
+                           cmd, lvm_error ( errno), lv_name_ptr);
          return LVM_ELVCREATE_LV_OPEN;
       }
    
       memset ( buf, 0, sizeof ( buf));
       if ( opt_v > 0) printf ( "%s -- initializing logical volume \"%s\"\n",
-                               cmd, lv_name);
+                               cmd, lv_name_ptr);
       if ( write ( lv_handle, buf, sizeof ( buf)) != sizeof ( buf)) {
          close ( lv_handle);
-         fprintf ( stderr, "%s -- ERROR %d initializing logical "
-                           "volume \"%s\"\n\n", cmd, errno, lv_name);
+         fprintf ( stderr, "%s -- ERROR \"%s\" initializing logical "
+                           "volume \"%s\"\n\n",
+                           cmd, lvm_error ( errno), lv_name_ptr);
          return LVM_ELVCREATE_LV_WRITE;
       }
       fsync ( lv_handle);
       close ( lv_handle);
    } else {
-      printf ( "%s -- WARNING: you are not initializing logical "
-               "volume \"%s\"\n", cmd, lv_name);
+      if ( opt_s == 0)
+         printf ( "%s -- WARNING: you are not initializing logical "
+                  "volume \"%s\"\n", cmd, lv_name_ptr);
    }
 
    if ( opt_v > 0) printf ( "%s -- changing lvmtab\n", cmd);
@@ -645,13 +839,16 @@ int main ( int argc, char **argv) {
         opt_A > 0) {
       printf ( "%s -- doing automatic backup of \"%s\"\n", cmd, vg_name);
       vg_cfgbackup ( vg_name, VG_BACKUP_DIR, opt_v, vg);
+   } else {
+      printf ( "%s -- WARNING: you don't have an automatic backup of \"%s\"\n",
+               cmd, vg_name);
    }
 
    lvm_interrupt ();
    LVM_UNLOCK ( 0);
 
    printf ( "%s -- logical volume \"%s\" successfully created\n\n",
-             cmd, lv_name);
+             cmd, lv_name_ptr);
 
    return 0;
 }
