@@ -4,7 +4,8 @@
  * Copyright (C) 1998 - 1999  Heinz Mauelshagen, Germany
  *
  * July,September 1998
- * February 1999
+ * February,October,November 1999
+ * February 2000
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +18,7 @@
  * GNU General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * along with LVM; see the file COPYING.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -27,6 +28,10 @@
  * Changelog
  *
  *    21/02/1999 - removed LVM_LOCK and LVM_UNLOCK
+ *    06/10/1999 - implemented support for long options
+ *    03/11/1999 - implemented stdout support
+ *                 (Thanks to Torsten Neumann <torsten@londo.rhein-main.de>)
+ *    15/02/2000 - use lvm_error()
  *
  */
 
@@ -40,7 +45,7 @@ char *cmd = NULL;
 int main ( int argc, char **argv) {
    int c = 0;
    int l = 0;
-   int le = 0;
+   ulong le = 0;
    int opt_v = 0;
    int ret = 0;
    int v = 0;
@@ -49,6 +54,14 @@ int main ( int argc, char **argv) {
 #else
    char *options = "h?v";
 #endif
+   struct option long_options[] = {
+#ifdef DEBUG
+      { "debug",   no_argument, NULL, 'd'},
+#endif
+      { "help",    no_argument, NULL, 'h'},
+      { "verbose", no_argument, NULL, 'v'},
+      { NULL, 0, NULL, 0}
+   };
    char *logfile = NULL;
    char *vg_name = NULL;
    char **vg_name_ptr = NULL;
@@ -60,7 +73,8 @@ int main ( int argc, char **argv) {
    SUSER_CHECK;
    LVMTAB_CHECK;
 
-   while ( ( c = getopt ( argc, argv, options)) != EOF) {
+   while ( ( c = getopt_long ( argc, argv, options,
+                               long_options, NULL)) != EOF) {
       switch ( c) {
 #ifdef DEBUG
          case 'd':
@@ -74,18 +88,18 @@ int main ( int argc, char **argv) {
 
          case 'h':
          case '?':
-            printf ( "\n%s\n\n%s -- Logical Volume Manager System "
+            printf ( "\n%s (IOP %d)\n\n%s -- Logical Volume Manager System "
                      "Activity Data Collector\n\n"
                      "Synopsis:\n"
                      "---------\n\n"
                      "%s\n"
 #ifdef DEBUG
-                     "\t[-d]\n"
+                     "\t[-d/--debug]\n"
 #endif
-                     "\t[-h/-?]\n"
-                     "\t[-v]\n"
-                     "\tLogFilePath\n\n",
-                     lvm_version, cmd, cmd);
+                     "\t[-h/-?/--help]\n"
+                     "\t[-v/--verbose]\n"
+                     "\t[LogFilePath]\n\n",
+                     lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd);
             return 0;
             break;
 
@@ -109,18 +123,17 @@ int main ( int argc, char **argv) {
 
 
    if ( argc - optind != 1) {
-      fprintf ( stderr, "%s -- please enter an activity data log file\n\n",
-                        cmd);
-      return LVM_ELVMSADC_NO_LOG_FILE;
-   } else logfile = argv[optind];
-
-   if ( opt_v > 0) printf ( "%s -- opening logfile \"%s\"\n", cmd, logfile);
-   if ( ( log = fopen ( logfile, "a")) == NULL) {
-      fprintf ( stderr, "%s -- ERROR %d opening \"%s\" for append\n\n",
-                        cmd, errno, logfile);
-      return LVM_ELVMSADC_FOPEN;
+      log = stdout;
+   } else {
+      logfile = argv[optind];
+      if ( opt_v > 0) printf ( "%s -- opening logfile \"%s\"\n", cmd, logfile);
+      if ( ( log = fopen ( logfile, "a")) == NULL) {
+         fprintf ( stderr, "%s -- ERROR \"%s\" opening \"%s\" for append\n\n",
+                           cmd, lvm_error ( errno), logfile);
+         return LVM_ELVMSADC_FOPEN;
+      }
+      chmod ( logfile, 0640);
    }
-   chmod ( logfile, 0640);
 
    if ( opt_v > 0) printf ( "%s -- finding all volume group(s)\n", cmd);
    vg_name_ptr = lvm_tab_vg_check_exist_all_vg ();
@@ -141,14 +154,14 @@ int main ( int argc, char **argv) {
    for ( ; optind < argc; optind++) {
       vg_name = argv[optind];
       if ( vg_check_active ( vg_name) != TRUE) continue;
-      if ( ( ret = vg_status_with_pv_and_lv ( vg_name, &vg, TRUE)) == 0) {
+      if ( ( ret = vg_status_with_pv_and_lv ( vg_name, &vg)) == 0) {
          for ( l = 0; l < vg->lv_max; l++) {
             if ( vg->lv[l] != NULL) {
                fprintf ( log, "%s %d\n",
                               vg->lv[l]->lv_name,
                               vg->lv[l]->lv_allocated_le);
                for ( le = 0; le < vg->lv[l]->lv_allocated_le; le++) {
-                  fprintf ( log, "%d %d %d\n",
+                  fprintf ( log, "%lu %u %u\n",
                                  le,
                                  vg->lv[l]->lv_current_pe[le].reads,
                                  vg->lv[l]->lv_current_pe[le].writes);
@@ -158,11 +171,13 @@ int main ( int argc, char **argv) {
       }
    }
 
-   if ( opt_v > 0) printf ( "%s -- closing logfile \"%s\"\n", cmd, logfile);
-   if ( fclose ( log) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d closing \"%s\"\n\n",
-                        cmd, errno, logfile);
-      return LVM_ELVMSADC_FCLOSE;
+   if ( logfile != NULL) {
+      if ( opt_v > 0) printf ( "%s -- closing logfile \"%s\"\n", cmd, logfile);
+      if ( fclose ( log) < 0) {
+         fprintf ( stderr, "%s -- ERROR \"%s\" closing \"%s\"\n\n",
+                           cmd, lvm_error ( errno), logfile);
+         return LVM_ELVMSADC_FCLOSE;
+      }
    }
 
    return 0;
