@@ -5,7 +5,8 @@
  *
  * June,October 1997
  * April,May,July,September 1998
- * January 1999
+ * January,October 1999
+ * February 2000
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +19,7 @@
  * GNU General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * along with LVM; see the file COPYING.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -34,6 +35,8 @@
  *    05/07/1998 - support LE/PV move (on same PV too)
  *    05/09/1998 - corrected some messages
  *    26/01/1999 - made volume group directory prefix a preprocessor option
+ *    06/10/1999 - implemented support for long options
+ *    15/02/2000 - use lvm_error()
  *
  */
 
@@ -56,6 +59,7 @@ int main ( int argc, char **argv) {
    int lv_num = 0;
    int np = 0;
    int opt_A = 1;
+   int opt_A_set = 0;
    int opt_f = 0;
    int opt_n = 0;
    int opt_t = 0;
@@ -64,9 +68,9 @@ int main ( int argc, char **argv) {
    int dst_pv_index = 0;
    int src_pv_index = 0;
    int vg_active = FALSE;
-   int *les = NULL;
-   int *source_pes = NULL;
-   int *dest_pes = NULL;
+   long *les = NULL;
+   long *source_pes = NULL;
+   long *dest_pes = NULL;
    ulong pe_moved = 0;
    char lv_name_buf[NAME_LEN] = { 0, };
    char *buffer = NULL;
@@ -76,6 +80,18 @@ int main ( int argc, char **argv) {
 #else
    char *options = "A:fh?n:tv";
 #endif
+   struct option long_options[] = {
+      { "autobackup", required_argument, NULL, 'A'},
+#ifdef DEBUG
+      { "debug",      no_argument,       NULL, 'd'},
+#endif
+      { "force",      no_argument,       NULL, 'f'},
+      { "help",       no_argument,       NULL, 'h'},
+      { "name",       required_argument, NULL, 'n'},
+      { "test",       no_argument,       NULL, 't'},
+      { "verbose",    no_argument,       NULL, 'v'},
+      { NULL, 0, NULL, 0}
+   };
    char **pv_allowed = NULL;
    char *src_pv_name = NULL;
    char *vg_name = NULL;
@@ -88,9 +104,11 @@ int main ( int argc, char **argv) {
    SUSER_CHECK;
    LVMTAB_CHECK;
 
-   while ( ( c = getopt ( argc, argv, options)) != EOF) {
+   while ( ( c = getopt_long ( argc, argv, options,
+                               long_options, NULL)) != EOF) {
       switch ( c) {
          case 'A':
+            opt_A_set++;
             if ( opt_A > 1) {
                fprintf ( stderr, "%s -- A option already given\n\n", cmd);
                return LVM_EINVALID_CMD_LINE;
@@ -124,26 +142,26 @@ int main ( int argc, char **argv) {
 
          case 'h':
          case '?':
-            printf ( "\n%s\n\n%s -- Physical Volume Move\n\n"
+            printf ( "\n%s  (IOP %d)\n\n%s -- Physical Volume Move\n\n"
                      "Synopsis:\n"
                      "---------\n\n"
                      "%s\n"
-                     "\t[-A y/n]\n"
+                     "\t[-A/--autobackup y/n]\n"
 #ifdef DEBUG
-                     "\t[-d]\n"
+                     "\t[-d/--debug]\n"
 #endif
-                     "\t[-f]\n"
-                     "\t[-h/?]\n"
-                     "\t[-n LogicalVolumePath[:LogicalExtend"
+                     "\t[-f/--force]\n"
+                     "\t[-h/-?/--help]\n"
+                     "\t[-n/--name LogicalVolumePath[:LogicalExtend"
                      "[-LogicalExtend]...]] /\n"
                      "\tSourcePhysicalVolumePath[:PhysicalExtend"
                      "[-PhysicalExtend]...]\n"
-                     "\t[-t]\n"
-                     "\t[-v[v]]\n"
+                     "\t[-t/--test]\n"
+                     "\t[-v[v]/--verbose [--verbose]]\n"
                      "\t[DestinationPhysicalVolumePath"
                      "[:PhysicalExtend[-PhysicalExtend]...]...]"
                      "\n\n",
-                     lvm_version, cmd, cmd);
+                     lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd);
             return 0;
             break;
 
@@ -183,9 +201,11 @@ int main ( int argc, char **argv) {
    }
 
    CMD_MINUS_CHK;
+   CMD_CHECK_OPT_A_SET;
+
 
    if ( opt_n == 0 && optind == argc) {
-      fprintf ( stderr, "%s -- please enter a source physical volume path",
+      fprintf ( stderr, "%s -- please enter a source physical volume path\n\n",
                         cmd);
       return LVM_EPVMOVE_PV_SOURCE;
    }
@@ -227,9 +247,9 @@ int main ( int argc, char **argv) {
                            "doesn't exist\n\n",
                            cmd, src_pv_name);
       } else {
-         fprintf ( stderr, "%s -- ERROR %d reading source physical "
+         fprintf ( stderr, "%s -- ERROR \"%s\" reading source physical "
                            "volume \"%s\"\n\n",
-                           cmd, ret, src_pv_name);
+                           cmd, lvm_error ( ret), src_pv_name);
       }
       return LVM_EPVMOVE_PV_READ;
    }
@@ -237,9 +257,9 @@ int main ( int argc, char **argv) {
    vg_name = pv_this.vg_name;
 
    if ( ( ret = pv_check_consistency ( &pv_this)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d:  physical volume \"%s\" "
+      fprintf ( stderr, "%s -- ERROR \"%s\" physical volume \"%s\" "
                         "is inconsistent\n\n",
-                        cmd, ret, src_pv_name);
+                        cmd, lvm_error ( ret), src_pv_name);
       return LVM_EPVMOVE_PV_CHECK_CONSISTENCY;
    }
 
@@ -256,9 +276,9 @@ int main ( int argc, char **argv) {
                             "from lvmtab\n",
                             cmd, vg_name);
    if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name, &vg)) != 0) {
-      fprintf ( stderr, "%s -- ERROR %d: couldn't get data of volume "
+      fprintf ( stderr, "%s -- ERROR \"%s\" couldn't get data of volume "
                         "group \"%s\"\n\n",
-                        cmd, ret, vg_name);
+                        cmd, lvm_error ( ret), vg_name);
       return LVM_EPVMOVE_VG_READ;
    }
 
@@ -266,19 +286,12 @@ int main ( int argc, char **argv) {
                             "of \"%s\"\n",
                             cmd, vg_name);
    if ( ( ret = vg_check_consistency_with_pv_and_lv ( vg)) != 0) {
-      fprintf ( stderr, "%s -- ERROR %d: volume group \"%s\" is "
+      fprintf ( stderr, "%s -- ERROR \"%s\" volume group \"%s\" is "
                         "inconsistent\n\n",
-                        cmd, ret, vg_name);
+                        cmd, lvm_error ( ret), vg_name);
       return LVM_EPVMOVE_VG_CHECK_CONSISTENCY;
    }
 
-   if ( vg->pv_cur == 1) {
-      fprintf ( stderr, "%s -- volume group \"%s\" only has 1 "
-                        "physical volume\n\n",
-                        cmd, vg_name);
-      return LVM_EPVMOVE_PV_CUR;
-   }
-
    /* get index of source physical volume */
    if ( opt_v > 0) printf ( "%s -- searching for source physical volume \"%s\" "
                             "in volume group \"%s\"\n",
@@ -355,16 +368,16 @@ int main ( int argc, char **argv) {
                                  "%s doesn't exist\n\n",
                                  cmd, argv[i]);
             } else {
-               fprintf ( stderr, "%s -- ERROR %d reading destination physical "
-                                 "volume \"%s\"\n\n",
-                                 cmd, ret, argv[i]);
+               fprintf ( stderr, "%s -- ERROR \"%s\" reading destination "
+                                 "physical volume \"%s\"\n\n",
+                                 cmd, lvm_error ( ret), argv[i]);
             }
             return LVM_EPVMOVE_PV_READ;
          }
          if ( ( ret = pv_check_consistency ( pv)) < 0) {
-            fprintf ( stderr, "%s -- ERROR %d:  physical volume \"%s\" "
+            fprintf ( stderr, "%s -- ERROR \"%s\" physical volume \"%s\" "
                               "is inconsistent\n\n",
-                              cmd, ret, argv[i]);
+                              cmd, lvm_error ( ret), argv[i]);
             return LVM_EPVMOVE_PV_CHECK_CONSISTENCY;
          }
          if ( pv_check_in_vg ( vg, pv->pv_name) == FALSE) {
@@ -400,7 +413,7 @@ int main ( int argc, char **argv) {
          for ( ext = 0; les[ext] != -1; ext++) {
             if ( les[ext] < 0 ||
                  les[ext] >= vg->lv[lv_index]->lv_current_le) {
-               fprintf ( stderr, "%s -- ERROR: logical extent %d invalid for "
+               fprintf ( stderr, "%s -- ERROR: logical extent %ld invalid for "
                                  "logical volume \"%s\"\n\n",
                                  cmd, les[ext], lv_name);
                return LVM_EPVMOVE_LE_INVALID;
@@ -421,7 +434,7 @@ int main ( int argc, char **argv) {
          if ( source_pes[ext] < 0 ||
               source_pes[ext] >= vg->pv[src_pv_index]->pe_total ||
               vg->pv[src_pv_index]->pe[source_pes[ext]].lv_num == 0) {
-            fprintf ( stderr, "%s -- ERROR: source physical extent %d invalid "
+            fprintf ( stderr, "%s -- ERROR: source physical extent %ld invalid "
                               "for physical volume \"%s\"\n\n",
                               cmd, source_pes[ext],
                               vg->pv[src_pv_index]->pv_name);
@@ -443,7 +456,7 @@ int main ( int argc, char **argv) {
          if ( dest_pes[ext] < 0 ||
               dest_pes[ext] >= vg->pv[dst_pv_index]->pe_total ||
               vg->pv[dst_pv_index]->pe[dest_pes[ext]].lv_num != 0) {
-            fprintf ( stderr, "%s -- ERROR: destination physical extent %d "
+            fprintf ( stderr, "%s -- ERROR: destination physical extent %ld "
                               "already in use on physical volume \"%s\"\n\n",
                               cmd, dest_pes[ext],
                               vg->pv[dst_pv_index]->pv_name);
@@ -516,8 +529,8 @@ int main ( int argc, char **argv) {
             printf ( "%s -- no physical extents to move\n\n", cmd);
             ret = 0;
          } else {
-            fprintf ( stderr, "%s -- ERROR %d moving physical extents\n\n",
-                              cmd, ret);
+            fprintf ( stderr, "%s -- ERROR \"%s\" moving physical extents\n\n",
+                              cmd, lvm_error ( ret));
          }
          return LVM_EPVMOVE_PV_MOVE_PES;
       }
@@ -538,6 +551,10 @@ int main ( int argc, char **argv) {
          printf ( "%s -- doing automatic backup of volume group \"%s\"\n",
                   cmd, vg_name);
          vg_cfgbackup ( vg_name, VG_BACKUP_DIR, opt_v, vg);
+      } else {
+         printf ( "%s -- WARNING: you don't have an automatic "
+                  "backup of \"%s\"\n",
+                  cmd, vg_name);
       }
    }
 
