@@ -1,11 +1,12 @@
 /*
  * tools/vgchange.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Germany
  *
  * March-May,October 1997
  * May,June,September 1998
- * February 1999
+ * February,July,October,December 1999
+ * February 2000
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,6 +36,12 @@
  *    27/06/1998 - changed lvm_tab_* calling convention
  *    20/09/1998 - changed message in deativation volume group
  *                 with open logical volumes
+ *    03/08/1999 - avoided snapshot logical volumes during activation
+ *    06/10/1999 - implemented support for long options
+ *    25/12/1999 - check for vg_check_online_all_pv() error in case
+ *                 pv_create_name_from_kdev_t() wasn't able to get a name,
+ *                 because /proc isn't mounted
+ *    15/02/2000 - use lvm_error()
  *
  */
 
@@ -63,8 +70,21 @@ int main ( int argc, char **argv) {
 #else
    char *options = "A:a:h?l:vx:";
 #endif
+   struct option long_options[] = {
+      { "autobackup",    required_argument, NULL, 'A'},
+      { "available",     required_argument, NULL, 'a'},
+#ifdef DEBUG
+      { "debug",         no_argument,       NULL, 'd'},
+#endif
+      { "help",          no_argument,       NULL, 'h'},
+      { "logicalvolume", required_argument, NULL, 'l'},
+      { "verbose",       no_argument,       NULL, 'v'},
+      { "allocation",    required_argument, NULL, 'x'},
+      { NULL, 0, NULL, 0}
+   };
    char **vg_name_ptr = NULL;
    int opt_A = 1;
+   int opt_A_set = 0;
    int opt_a = 0;
    int opt_a_action = 0;
    int opt_l = 0;
@@ -82,9 +102,11 @@ int main ( int argc, char **argv) {
    SUSER_CHECK;
    LVMTAB_CHECK;
 
-   while ( ( c = getopt ( argc, argv, options)) != EOF) {
+   while ( ( c = getopt_long ( argc, argv, options,
+                               long_options, NULL)) != EOF) {
       switch ( c) {
          case 'A':
+            opt_A_set++;
             if ( opt_A > 1) {
                fprintf ( stderr, "%s -- A option already given\n\n", cmd);
                return LVM_EINVALID_CMD_LINE;
@@ -129,20 +151,20 @@ int main ( int argc, char **argv) {
 
          case 'h':
          case '?':
-            printf ( "\n%s\n\n%s -- Volume Group Change\n\n"
+            printf ( "\n%s  (IOP %d)\n\n%s -- Volume Group Change\n\n"
                      "Synopsis:\n"
                      "---------\n\n"
                      "%s\n"
-                     "\t[-A y/n]\n"
-                     "\t[-a y/n [VolumeGroupName...]]\n"
+                     "\t[-A/--autobackup y/n]\n"
+                     "\t[-a/--available y/n [VolumeGroupName...]]\n"
 #ifdef DEBUG
-                     "\t[-d]\n"
+                     "\t[-d/--debug]\n"
 #endif
-                     "\t[-h/-?]\n"
-                     "\t[-l MaxLogicalVolumes]\n"
-                     "\t[-v]\n"
-                     "\t[-x y/n [VolumeGroupName...]]\n\n",
-                     lvm_version, cmd, cmd);
+                     "\t[-h/-?/--help]\n"
+                     "\t[-l/--logicalvolume MaxLogicalVolumes]\n"
+                     "\t[-v/--verbose]\n"
+                     "\t[-x/--allocation y/n [VolumeGroupName...]]\n\n",
+                     lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd);
             return 0;
             break;
 
@@ -200,6 +222,8 @@ int main ( int argc, char **argv) {
    }
   
    CMD_MINUS_CHK;
+   CMD_CHECK_OPT_A_SET;
+
 
    if ( opt_a + opt_l + opt_x == 0) {
       fprintf ( stderr, "%s -- you have to give the -a, -l or the -x "
@@ -275,9 +299,8 @@ int main ( int argc, char **argv) {
                               "with invalid version\n",
                       cmd, vg_name);
          else
-            fprintf ( stderr, "%s -- ERROR %d checking existence of "
-                              "volume group \"%s\"\n",
-                      cmd, ret, vg_name);
+            fprintf ( stderr, "%s -- volume group \"%s\" does not exist\n",
+                              cmd, vg_name);
          vg_deactivate ( vg_name);
          continue;
       }
@@ -288,15 +311,16 @@ int main ( int argc, char **argv) {
          vg_free ( vg, FALSE);
          vg = NULL;
       }
+
       if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name, &vg)) < 0) {
          if ( ret == -LVM_EVG_CFGRESTORE_OPEN) {
             fprintf ( stderr, "%s -- ERROR: volume group \"%s\" "
                               "doesn't exist\n",
                               cmd, vg_name);
          } else {
-            fprintf ( stderr, "%s -- ERROR %d: reading lvmtab data of "
+            fprintf ( stderr, "%s -- ERROR \"%s\" reading lvmtab data of "
                               "volume group \"%s\"\n",
-                              cmd, ret, vg_name);
+                              cmd, lvm_error ( ret), vg_name);
          }
          if ( vg_check_active ( vg_name) == TRUE) vg_deactivate ( vg_name);
          continue;
@@ -305,9 +329,9 @@ int main ( int argc, char **argv) {
       if ( opt_v > 0) printf ( "%s -- checking volume group consistency "
                                " of \"%s\"\n", cmd, vg_name);
       if ( ( ret = vg_check_consistency_with_pv_and_lv ( vg)) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d: volume group \"%s\" "
+         fprintf ( stderr, "%s -- ERROR \"%s\" volume group \"%s\" "
                            " is inconsistent\n",
-                           cmd, ret, vg_name);
+                           cmd, lvm_error ( ret), vg_name);
          vg_deactivate ( vg_name);
          continue;
       }
@@ -319,35 +343,67 @@ int main ( int argc, char **argv) {
             if ( opt_v > 0) printf ( "%s -- checking if all physical volumes "
                                      "of volume group \"%s\" are available\n",
                                      cmd, vg_name);
-            if ( ( ret = vg_check_online_all_pv ( vg, &pv_offl,
-                                                  &pv_incons)) < 0 &&
-                 ret != -LVM_EPV_READ_MD_DEVICE) {
-               if ( pv_offl != NULL) for ( p = 0; pv_offl[p] != NULL; p++) {
-                  fprintf ( stderr, "%s -- ERROR: physical volume \"%s\" of "
-                                    "volume group \"%s\" is offline\n",
-                                    cmd, pv_offl[p]->pv_name, vg_name);
+            if ((ret = vg_check_online_all_pv(vg, &pv_offl, &pv_incons)) < 0 &&
+		ret != -LVM_EPV_READ_MD_DEVICE) {
+               if ( ret == -LVM_EPV_READ_PV_CREATE_NAME_FROM_KDEV_T) {
+                     fprintf ( stderr, "%s -- ERROR: can't get name(s) of "
+                                       "physical volumes\n"
+                                       "%s -- Please check, if /proc "
+                                       "is mounted\n",
+                                       cmd, cmd);
+               } else if ( ret != -LVM_EPV_READ_MD_DEVICE) {
+                  if ( pv_offl != NULL) for ( p = 0; pv_offl[p] != NULL; p++) {
+                     fprintf ( stderr, "%s -- ERROR: physical volume \"%s\" of "
+                                       "volume group \"%s\" is offline\n",
+                                       cmd, pv_offl[p]->pv_name, vg_name);
+                  }
+                  if ( pv_incons != NULL) for ( p = 0; pv_incons[p] != NULL; p++) {
+                     fprintf ( stderr, "%s -- ERROR: physical volume \"%s\" of "
+                                       "volume group \"%s\" is inconsistent\n",
+                                       cmd, pv_incons[p]->pv_name, vg_name);
+                  }
+                  printf ( "%s -- run vgscan\n", cmd);
                }
-               if ( pv_incons != NULL) for ( p = 0; pv_incons[p] != NULL; p++) {
-                  fprintf ( stderr, "%s -- ERROR: physical volume \"%s\" of "
-                                    "volume group \"%s\" is inconsistent\n",
-                                    cmd, pv_incons[p]->pv_name, vg_name);
-               }
-               printf ( "%s -- run vgscan\n", cmd);
+               if ( pv_offl   != NULL) free ( pv_offl);
+               if ( pv_incons != NULL) free ( pv_incons);
                continue;
             }
-            if ( pv_offl   != NULL) free ( pv_offl);
-            if ( pv_incons != NULL) free ( pv_incons);
       
+            /* Avoid snapshot logical volumes during activation */
+            for ( l = 0; l < vg->lv_max; l++) {
+               if ( vg->lv[l] != NULL) {
+                  vg->lv[l]->lv_block_exception = NULL;
+                  vg->lv[l]->lv_remap_ptr = 0;
+                  vg->lv[l]->lv_remap_end = 0;
+                  if ( vg->lv[l]->lv_access & LV_SNAPSHOT_ORG) {
+                     printf ( "%s -- resetting source snapshot logical "
+                              "volume \"%s\"\n", cmd, vg->lv[l]->lv_name);
+                     vg->lv[l]->lv_access &= ~LV_SNAPSHOT_ORG;
+                  }
+                  if ( vg->lv[l]->lv_access & LV_SNAPSHOT) {
+                     printf ( "%s -- resetting destination snapshot logical "
+                              "volume \"%s\"\n", cmd, vg->lv[l]->lv_name);
+                     vg->lv[l]->lv_access &= ~LV_SNAPSHOT;
+                  }
+               }
+            }
+
             if ( opt_v > 0) printf ( "%s -- creating VGDA for \"%s\" "
                                      "in kernel\n",
                                      cmd, vg_name);
-            if ( ( ret = vg_create ( vg_name, vg)) == 0)
+
+            ret = vg_create ( vg_name, vg);
+            if ( ret == -LVM_EVG_CREATE_REMOVE_OPEN) {
+               vg_create_dir_and_group( vg);
+               ret = vg_create ( vg_name, vg);
+            }
+            if ( ret == 0)
                printf ( "%s -- volume group \"%s\" successfully activated\n",
                         cmd, vg_name);
             else {
-   #ifdef DEBUG
+#ifdef DEBUG
                debug ( "%s -- vg_create returned: %d\n", cmd, ret);
-   #endif
+#endif
                if ( ret == -LVM_EVG_CREATE_REMOVE_OPEN) {
                   fprintf ( stderr, "%s -- can't open logical volume manager "
                                     "to activate volume group \"%s\"\n",
@@ -361,9 +417,9 @@ int main ( int argc, char **argv) {
                                     "volume group \"%s\"\n",
                                     cmd, vg_name);
                } else {
-                  fprintf ( stderr, "%s -- ERROR %d activating "
+                  fprintf ( stderr, "%s -- ERROR \"%s\" activating "
                                     "volume group \"%s\"\n",
-                                    cmd, ret, vg_name);
+                                    cmd, lvm_error ( ret), vg_name);
                }
                continue;
             }
@@ -375,9 +431,9 @@ int main ( int argc, char **argv) {
                printf ( "%s -- volume group \"%s\" successfully deactivated\n",
                         cmd, vg_name);
             else {
-   #ifdef DEBUG
+#ifdef DEBUG
                debug ( "%s -- vg_remove returned: %d\n", cmd, ret);
-   #endif
+#endif
                if ( ret == -LVM_EVG_CREATE_REMOVE_OPEN) {
                   fprintf ( stderr, "%s -- can't open logical volume manager "
                                     "to deactivate \"%s\"?\n", cmd, vg_name);
@@ -399,9 +455,9 @@ int main ( int argc, char **argv) {
                    fprintf ( stderr, "%s -- volume group \"%s\" isn't active\n",
                                      cmd, vg_name);
                } else {
-                   fprintf ( stderr, "%s -- ERROR %d DEactivating volume "
+                   fprintf ( stderr, "%s -- ERROR \"%s\" DEactivating volume "
                                      "group \"%s\"\n",
-                                     cmd, ret, vg_name);
+                                     cmd, lvm_error ( ret), vg_name);
                }
                continue;
             }
@@ -434,8 +490,9 @@ int main ( int argc, char **argv) {
             if ( opt_v > 0) printf ( "%s -- storing volume group data of "
                                      "%s on disk(s)\n", cmd, vg_name);
             if ( ( ret = vg_write_with_pv_and_lv ( vg)) < 0) {
-               fprintf ( stderr, "%s -- ERROR %d storing volume group data "
-                                 "of \"%s\" on disk(s)\n\n", cmd, ret, vg_name);
+               fprintf ( stderr, "%s -- ERROR \"%s\"storing volume group data "
+                                 "of \"%s\" on disk(s)\n\n",
+                                 cmd, lvm_error ( ret), vg_name);
                return LVM_EVGCHANGE_VG_WRITE;
             }
       
@@ -447,7 +504,8 @@ int main ( int argc, char **argv) {
                   ret = vg_clear_extendable ( vg_name);
                }
                if ( ret < 0) {
-                  fprintf ( stderr, "%s -- ERROR %d ", cmd, ret);
+                  fprintf ( stderr, "%s -- ERROR \"%s\" ",
+                                    cmd, lvm_error ( ret));
                   if ( set_extendable == TRUE) {
                       fprintf ( stderr, "en");
                   } else {
@@ -472,8 +530,9 @@ int main ( int argc, char **argv) {
                printf ( "%s -- volume group \"%s\" successfully changed\n\n",
                         cmd, vg_name);
             } else {
-               fprintf ( stderr, "%s -- ERROR %d storing volume group backup "
-                                 "of \"%s\" in %s\n\n", cmd, ret, vg_name, dir);
+               fprintf ( stderr, "%s -- ERROR \"%s\" storing volume group "
+                                 "backup of \"%s\" in %s\n\n",
+                                 cmd, lvm_error ( ret), vg_name, dir);
                return LVM_EVGCHANGE_VG_CFG_BACKUP;
             }
          }
@@ -542,9 +601,9 @@ int main ( int argc, char **argv) {
          }
 
          if ( ( ret = vg_write_with_pv_and_lv ( vg)) < 0) {
-            fprintf ( stderr, "%s -- ERROR %d: storing volume group data "
+            fprintf ( stderr, "%s -- ERROR \"%s\" storing volume group data "
                               "of \"%s\" on disks\n\n", 
-                              cmd, ret, vg_name);
+                              cmd, lvm_error ( ret), vg_name);
             return LVM_EVGCHANGE_VG_WRITE;
          }
 
@@ -554,14 +613,19 @@ int main ( int argc, char **argv) {
                         cmd, vg_name);
                if ( ( ret = vg_cfgbackup ( vg_name, VG_BACKUP_DIR,
                                            opt_v, vg)) < 0) {
-                  fprintf ( stderr, "%s -- ERROR %d writing VG backup of "
-                                    "\"%s\"\n\n", cmd, ret, vg_name);
+                  fprintf ( stderr, "%s -- ERROR \"%s\" writing VG backup of "
+                                    "\"%s\"\n\n",
+                                    cmd, lvm_error ( ret), vg_name);
                   return LVM_EVGCHANGE_VG_CFGBACKUP;
                }
+            } else {
+               printf ( "%s -- WARNING: you don't have an automatic "
+                        "backup of \"%s\"\n",
+                        cmd, vg_name);
             }
          } else {
-            fprintf ( stderr, "%s -- ERROR %d writing \"%s\"\n\n",
-                              cmd, ret, LVMTAB);
+            fprintf ( stderr, "%s -- ERROR \"%s\" writing \"%s\"\n\n",
+                              cmd, lvm_error ( ret), LVMTAB);
             return LVM_EVGCHANGE_VG_CFGBACKUP_LVMTAB;
          }
 
@@ -569,23 +633,23 @@ int main ( int argc, char **argv) {
                                   "volume group \"%s\"\n",
                                   cmd, vg_name);
          if ( ( ret = vg_remove_dir_and_group_and_nodes ( vg_name)) < 0) {
-            fprintf ( stderr, "%s -- ERROR %d: removing volume group "
+            fprintf ( stderr, "%s -- ERROR \"%s\" removing volume group "
                               "nodes of \"%s\"\n\n",
-                              cmd, ret, vg_name);
+                              cmd, lvm_error ( ret), vg_name);
             return LVM_EVGCHANGE_VG_REMOVE_DIR_AND_GROUP_NODES;
          }
       
          if ( ( ret = vg_create_dir_and_group_and_nodes ( vg, opt_v)) < 0) {
-            fprintf ( stderr, "%s -- ERROR %d: creating volume group "
+            fprintf ( stderr, "%s -- ERROR \"%s\" creating volume group "
                               "nodes of \"%s\"\n\n",
-                              cmd, ret, vg_name);
+                              cmd, lvm_error ( ret), vg_name);
             return LVM_EVGCHANGE_VG_CREATE_DIR_AND_GROUP_NODES;
          }
       } /* if ( opt_l > 0) */
       lvm_interrupt ();
    }
 
-   printf ( "\n");
+   putchar ( '\n');
 
    if ( vg != NULL) vg_free ( vg, FALSE);
    LVM_UNLOCK ( 0);
