@@ -1,9 +1,11 @@
 /*
  * tools/vgck.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Germany
  *
  * June 1998
+ * October 1999
+ * February 2000
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,6 +24,14 @@
  *
  */
 
+/*
+ * Changelog
+ *
+ *    06/10/1999 - implemented support for long options
+ *    15/02/2000 - use lvm_error()
+ *
+ */
+
 #include <lvm_user.h>
 
 char *cmd = NULL;
@@ -42,6 +52,14 @@ int main ( int argc, char **argv) {
 #else
    char *options = "h?v";
 #endif
+   struct option long_options[] = {
+#ifdef DEBUG
+      { "debug",   no_argument, NULL, 'd'},
+#endif
+      { "help",    no_argument, NULL, 'h'},
+      { "verbose", no_argument, NULL, 'v'},
+      { NULL, 0, NULL, 0}
+   };
    char **vg_name_ptr = NULL;
    vg_t *vg = NULL;
 
@@ -50,7 +68,8 @@ int main ( int argc, char **argv) {
    SUSER_CHECK;
    LVMTAB_CHECK;
 
-   while ( ( c = getopt ( argc, argv, options)) != EOF) {
+   while ( ( c = getopt_long ( argc, argv, options,
+                               long_options, NULL)) != EOF) {
       switch ( c) {
 
 #ifdef DEBUG
@@ -65,17 +84,17 @@ int main ( int argc, char **argv) {
 
          case 'h':
          case '?':
-            printf ( "\n%s\n\n%s -- Volume Group Check\n\n"
+            printf ( "\n%s  (IOP %d)\n\n%s -- Volume Group Check\n\n"
                      "Synopsis:\n"
                      "---------\n\n"
                      "%s\n"
 #ifdef DEBUG
-                     "\t[-d]\n"
+                     "\t[-d/--debug]\n"
 #endif
-                     "\t[-h/-?]\n"
-                     "\t[-v]\n"
+                     "\t[-h/-?/--help]\n"
+                     "\t[-v/--verbose]\n"
                      "\t[VolumeGroupName...]\n\n",
-                     lvm_version, cmd, cmd);
+                     lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd);
             return 0;
             break;
 
@@ -150,9 +169,9 @@ int main ( int argc, char **argv) {
                               "with invalid version\n",
                       cmd, vg_name);
          else
-            fprintf ( stderr, "%s -- ERROR %d checking existence of "
+            fprintf ( stderr, "%s -- ERROR \"%s\" checking existence of "
                               "volume group \"%s\"\n",
-                              cmd, ret, vg_name);
+                              cmd, lvm_error ( ret), vg_name);
          fprintf ( stderr, "%s -- trying to deactivate volume group "
                            "\"%s\"\n", cmd, vg_name);
          vg_deactivate ( vg_name);
@@ -161,9 +180,9 @@ int main ( int argc, char **argv) {
       if ( opt_v > 0) printf ( "%s -- reading volume group data for \"%s\" "
                                "from lvmtab\n", cmd, vg_name);
       if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name, &vg)) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d: reading lvmtab data of "
+         fprintf ( stderr, "%s -- ERROR \"%s\" reading lvmtab data of "
                            "volume group \"%s\"\n",
-                           cmd, ret, vg_name);
+                           cmd, lvm_error ( ret), vg_name);
          err_lvmtab = 1;
          fprintf ( stderr, "%s -- trying to deactivate volume group "
                            "\"%s\"\n", cmd, vg_name);
@@ -172,8 +191,9 @@ int main ( int argc, char **argv) {
          if ( opt_v > 0) printf ( "%s -- checking volume group consistency "
                                   " of \"%s\" in lvmtab\n", cmd, vg_name);
          if ( ( ret = vg_check_consistency_with_pv_and_lv ( vg)) < 0) {
-            fprintf ( stderr, "%s -- ERROR %d: VGDA of \"%s\" is INCONSISTENT "
-                              "in lvmtab\n", cmd, ret, vg_name);
+            fprintf ( stderr, "%s -- ERROR \"%s\" VGDA of \"%s\" is "
+                              "INCONSISTENT in lvmtab\n",
+                              cmd, lvm_error ( ret), vg_name);
             fprintf ( stderr, "%s -- trying to deactivate volume group "
                               "\"%s\"\n", cmd, vg_name);
             vg_deactivate ( vg_name);
@@ -188,9 +208,9 @@ int main ( int argc, char **argv) {
       if ( opt_v > 0) printf ( "%s -- reading volume group data for \"%s\" "
                                "from physical volume(s)\n", cmd, vg_name);
       if ( ( ret = vg_read_with_pv_and_lv ( vg_name, &vg)) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d: reading data of volume "
+         fprintf ( stderr, "%s -- ERROR \"%s\" reading data of volume "
                            "group \"%s\" from physical volume(s)\n",
-                           cmd, ret, vg_name);
+                           cmd, lvm_error ( ret), vg_name);
          if ( err_lvmtab == 0) {
             fprintf ( stderr, "%s -- please DON'T run vgscan prior to "
                               "vgcfgrestore\n", cmd);
@@ -205,9 +225,9 @@ int main ( int argc, char **argv) {
                      cmd, vg_name, vg->pv_cur > 1 ? "s" : "");
          }
          if ( ( ret = vg_check_consistency_with_pv_and_lv ( vg)) < 0) {
-            fprintf ( stderr, "%s -- ERROR %d: VGDA of \"%s\" is INCONSISTENT "
-                              "on physical volume%s\n",
-                              cmd, ret, vg_name,
+            fprintf ( stderr, "%s -- ERROR \"%s\" VGDA of \"%s\" is "
+                              "INCONSISTENT on physical volume%s\n",
+                              cmd, lvm_error ( ret), vg_name,
                               vg->pv_cur > 1 ? "s" : "");
             fprintf ( stderr, "%s -- trying to deactivate volume group "
                               "\"%s\"\n", cmd, vg_name);
