@@ -1,10 +1,12 @@
 /*
  * tools/vgremove.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Germany
  *
  * February, October 1997
  * May-June,September 1998
+ * September-October 1999
+ * February 2000
  *
  * vgremove is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,6 +35,9 @@
  *                 instead of vg_read() and pv_read_all_pv_of_vg()
  *    27/06/1998 - changed lvm_tab_* calling convention
  *    05/09/1998 - corrected some messages
+ *    25/09/1999 - added new exit codes
+ *    06/10/1999 - implemented support for long options
+ *    15/02/2000 - use lvm_error()
  *
  */
 
@@ -55,16 +60,24 @@ int main ( int argc, char **argv)
 #else
    char *options = "dh?v";
 #endif
+   struct option long_options[] = {
+#ifdef DEBUG
+      { "debug",   no_argument, NULL, 'd'},
+#endif
+      { "help",    no_argument, NULL, 'h'},
+      { "verbose", no_argument, NULL, 'v'},
+      { NULL,      0,           NULL, 0},
+   };
    char *vg_name = NULL;
    vg_t *vg = NULL;
-   pv_t **pv = NULL;
 
    cmd = basename ( argv[0]);
 
    SUSER_CHECK;
    LVMTAB_CHECK;
 
-   while ( ( c = getopt ( argc, argv, options)) != EOF) {
+   while ( ( c = getopt_long ( argc, argv, options,
+                               long_options, NULL)) != EOF) {
       switch ( c) {
 #ifdef DEBUG
          case 'd':
@@ -78,17 +91,17 @@ int main ( int argc, char **argv)
 
          case 'h':
          case '?':
-            printf ( "\n%s\n\n%s -- Volume Group Remove\n\n"
+            printf ( "\n%s  (IOP %d)\n\n%s -- Volume Group Remove\n\n"
                      "Synopsis:\n"
                      "---------\n\n"
                      "%s\n"
 #ifdef DEBUG
-                     "\t[-d]\n"
+                     "\t[-d/--debug]\n"
 #endif
-                     "\t[-h/?]\n"
-                     "\t[-v]\n"
+                     "\t[-h/-?/--help]\n"
+                     "\t[-v/--verbose]\n"
                      "\tVolumeGroupName [VolumeGroupName...]\n\n",
-                     lvm_version, cmd, cmd);
+                     lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd);
             return 0;
             break;
 
@@ -121,18 +134,18 @@ int main ( int argc, char **argv)
       if ( opt_v > 0) printf ( "%s -- checking volume group name \"%s\"\n",
                               cmd, vg_name);
       if ( vg_check_name ( vg_name) < 0) {
-         fprintf ( stderr, "%s -- ERROR: invalid volume group name \"%s\"\n",
+         fprintf ( stderr, "%s -- ERROR: invalid volume group name \"%s\"\n\n",
                            cmd, vg_name);
-         continue;
+         return LVM_EVGREMOVE_VG_CHECK_NAME;
       }
    
       if ( opt_v > 0) printf ( "%s -- checking volume group \"%s\" activity\n",
                                cmd, vg_name);
       if ( vg_check_active ( vg_name) == TRUE) {
          fprintf ( stderr, "%s -- ERROR: can't remove active volume "
-                           "group \"%s\"\n",
+                           "group \"%s\"\n\n",
                            cmd, vg_name);
-         continue;
+         return LVM_EVGREMOVE_VG_CHECK_ACTIVE;
       }
    
       /* does VG exist? */
@@ -141,27 +154,28 @@ int main ( int argc, char **argv)
       if ( ( ret = lvm_tab_vg_check_exist ( vg_name, NULL)) != TRUE) {
          if ( ret == -LVM_EVG_CHECK_EXIST_PV_COUNT) {
             fprintf ( stderr, "%s -- ERROR: not all physical volumes of "
-                              "volume group \"%s\" are online\n",
+                              "volume group \"%s\" are online\n\n",
                               cmd, vg_name);
+            return LVM_EVGREMOVE_PV_ONLINE;
          } else if ( ret == FALSE) {
-            fprintf ( stderr, "%s -- volume group \"%s\" doesn't exist\n",
+            fprintf ( stderr, "%s -- volume group \"%s\" doesn't exist\n\n",
                               cmd, vg_name);
          } else {
-            fprintf ( stderr, "%s -- ERROR %d: checking existence of "
-                              "volume group \"%s\"\n",
-                              cmd, ret, vg_name);
+            fprintf ( stderr, "%s -- ERROR \"%s\" checking existence of "
+                              "volume group \"%s\"\n\n",
+                              cmd, lvm_error ( ret), vg_name);
          }
-         continue;
+         return LVM_EVGREMOVE_VG_ERROR;
       }
    
       if ( opt_v > 0) printf ( "%s -- reading data of volume group \"%s\" "
                                "from \"%s\"\n",
                                cmd, vg_name, LVMTAB);
-      if ( lvm_tab_vg_read_with_pv_and_lv ( vg_name, &vg) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d: can't get data of volume group "
-                           "\"%s\" from \"%s\"\n",
-                           cmd, ret, vg_name, LVMTAB);
-         continue;
+      if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name, &vg)) < 0) {
+         fprintf ( stderr, "%s -- ERROR \"%s\" can't get data of volume group "
+                           "\"%s\" from \"%s\"\n\n",
+                           cmd, lvm_error ( ret), vg_name, LVMTAB);
+         return LVM_EVGREMOVE_VG_READ;
       }
    
       if ( opt_v > 0) printf ( "%s -- checking for logical volumes in "
@@ -169,23 +183,23 @@ int main ( int argc, char **argv)
                                cmd, vg_name);
       if ( vg->lv_cur > 0) {
          fprintf ( stderr, "%s -- ERROR: can't remove volume group \"%s\" with"
-                           " %u logical volume%s\n",
+                           " %u logical volume%s\n\n",
                            cmd, vg_name, vg->lv_cur, vg->lv_cur > 1 ? "s" : "");
-         continue;
+         return LVM_EVGREMOVE_LV_EXISTS;
       }
    
       /* init physical volumes */
       if ( opt_v > 0) printf ( "%s -- initializing all physical volumes of "
-                               "volume group \"%s\"\n",
+                               "volume group \"%s\"\n\n",
                                cmd, vg_name);
       for ( p = 0; vg->pv[p] != NULL; p++) {
          if ( opt_v > 0) printf ( "%s -- initializing physical volume \"%s\"\n",
-                                  cmd, pv[p]->pv_name);
+                                  cmd, vg->pv[p]->pv_name);
          if ( ( ret = pv_setup_for_create ( vg->pv[p]->pv_name, vg->pv[p],
                                             vg->pv[p]->pv_size)) < 0) {
-            fprintf ( stderr, "%s -- ERROR %d initializing physical "
+            fprintf ( stderr, "%s -- ERROR \"%s\" initializing physical "
                               "volume \"%s\"\n\n",
-                              cmd, ret,  vg->pv[p]->pv_name);
+                              cmd, lvm_error ( ret), vg->pv[p]->pv_name);
             return LVM_EVGREMOVE_PV_SETUP;
          }
       }
@@ -195,16 +209,16 @@ int main ( int argc, char **argv)
                                "volumes of volume group \"%s\" on disk(s)\n",
                                cmd, vg_name);
       if ( ( ret = pv_write_all_pv_of_vg ( vg)) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d storing all %u initilized physical "
-                           "volumes of volume group \"%s\"\n\n",
-                           cmd, ret, vg->pv_cur, vg_name);
+         fprintf ( stderr, "%s -- ERROR \"%s\" storing all %u initialized "
+                           "physical volumes of volume group \"%s\"\n\n",
+                           cmd, lvm_error ( ret), vg->pv_cur, vg_name);
          return LVM_EVGREMOVE_PV_WRITE_ALL_PV_OF_VG;
       }
    
       if ( opt_v > 0) printf ( "%s -- removing special files of "
                                "volume group \"%s\"\n", cmd, vg_name);
       if ( vg_remove_dir_and_group_and_nodes ( vg_name) < 0) {
-         fprintf ( stderr, "%s -- ERROR remving special files of volume "
+         fprintf ( stderr, "%s -- ERROR removing special files of volume "
                            "group \"%s\"\n",
                            cmd, vg_name);
       }
@@ -213,9 +227,9 @@ int main ( int argc, char **argv)
                                "group \"%s\"\n",
                                cmd, LVMTAB, vg_name);
       if ( ( ret = lvm_tab_vg_remove ( vg_name)) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d removing volume group \"%s\" "
-                           "entry from \"%s\"\n",
-                           cmd, ret, vg_name, LVMTAB);
+         fprintf ( stderr, "%s -- ERROR \"%s\" removing volume group \"%s\" "
+                           "entry from \"%s\"\n\n",
+                           cmd, lvm_error ( ret), vg_name, LVMTAB);
          return LVM_EVGREMOVE_VG_REMOVE;
       }
    
